@@ -1,4 +1,4 @@
-/*	$Id: gshhs.h,v 1.20 2009/01/09 04:02:35 guru Exp $
+/*	$Id: gshhs.h,v 1.27 2009/06/25 22:23:38 guru Exp $
  *
  * Include file defining structures used in gshhs.c
  *
@@ -28,6 +28,10 @@
  *			  For use with version 1.6 of GSHHS which now has WDBII
  *			  borders and rivers.
  *	03-JUL-2008.  PW: Version 1.11. New -I<id> option to pull out a single pol
+ *	27-MAY-2009.  PW: Version 1.12. Now includes container polygon ID in header,
+ *			  an ancestor ID, and area of the reduced polygon. Works on
+ *			  GSHHS 2.0 data.
+ *			  Header is now 44 bytes (all 4-byte integers)
  */
 
 #ifndef _GSHHS
@@ -36,6 +40,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <ctype.h>
 #include <math.h>
 
 #ifdef WIN32
@@ -50,8 +55,9 @@
 #define SEEK_CUR 1
 #endif
 
-#define GSHHS_DATA_VERSION	6	/* For v1.5 data set */
-#define GSHHS_PROG_VERSION	"1.11"
+#define GSHHS_DATA_RELEASE	7	/* For v2.0 data set */
+#define GSHHS_DATA_VERSION	"2.0"	/* For v2.0 data set */
+#define GSHHS_PROG_VERSION	"1.12"
 
 #define GSHHS_SCL	1.0e-6	/* Convert micro-degrees to degrees */
 
@@ -60,17 +66,21 @@
 #define swabi4(i4) (((i4) >> 24) + (((i4) >> 8) & 65280) + (((i4) & 65280) << 8) + (((i4) & 255) << 24))
 
 struct GSHHS {	/* Global Self-consistent Hierarchical High-resolution Shorelines */
-	int id;				/* Unique polygon id number, starting at 0 */
-	int n;				/* Number of points in this polygon */
-	int flag;			/* = level + version << 8 + greenwich << 16 + source << 24 */
-	/* flag contains 4 items, one in each byte, as follows:
+	int id;		/* Unique polygon id number, starting at 0 */
+	int n;		/* Number of points in this polygon */
+	int flag;	/* = level + version << 8 + greenwich << 16 + source << 24 + river << 25 */
+	/* flag contains 5 items, as follows:
 	 * low byte:	level = flag & 255: Values: 1 land, 2 lake, 3 island_in_lake, 4 pond_in_island_in_lake
-	 * 2nd byte:	version = (flag >> 8) & 255: Values: Should be 4 for GSHHS version 1.4
-	 * 3rd byte:	greenwich = (flag >> 16) & 255: Values: Greenwich is 1 if Greenwich is crossed
-	 * 4th byte:	source = (flag >> 24) & 255: Values: 0 = CIA WDBII, 1 = WVS
+	 * 2nd byte:	version = (flag >> 8) & 255: Values: Should be 7 for GSHHS release 7
+	 * 3rd byte:	greenwich = (flag >> 16) & 1: Values: Greenwich is 1 if Greenwich is crossed
+	 * 4th byte:	source = (flag >> 24) & 1: Values: 0 = CIA WDBII, 1 = WVS
+	 * 4th byte:	river = (flag >> 25) & 1: Values: 0 = not set, 1 = river-lake and level = 2
 	 */
 	int west, east, south, north;	/* min/max extent in micro-degrees */
-	int area;			/* Area of polygon in 1/10 km^2 */
+	int area;	/* Area of polygon in 1/10 km^2 */
+	int area_full;	/* Area of original full-resolution polygon in 1/10 km^2 */
+	int container;	/* Id of container polygon that encloses this polygon (-1 if none) */
+	int ancestor;	/* Id of ancestor polygon in the full resolution set that was the source of this polygon (-1 if none) */
 };
 
 struct	POINT {	/* Each lon, lat pair is stored in micro-degrees in 4-byte integer format */
