@@ -1,6 +1,6 @@
-/* $Id: img2mercgrd.c,v 1.24 2007/09/30 04:16:49 guru Exp $
+/* $Id: img2mercgrd.c,v 1.31 2009/05/13 21:06:43 guru Exp $
  *
- * Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ * Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
  * See COPYING file for copying and redistribution conditions.
  *
  * img2mercgrd.c
@@ -92,9 +92,11 @@ struct IMG2MERCGRD_CTRL {
 
 int main (int argc, char **argv)
 {
-	int	i, ij, navgsq, error = FALSE, irarg = -1, tempint, n_files = 0;
-	int	navg;	/* navg by navg pixels are averaged if navg > 1; else if navg == 1 do nothing */
-	int	iout, jout, iinstart, iinstop, jinstart, jinstop, k, kk, ion, jin, jj, iin, ii, kstart;
+	GMT_LONG	i, navgsq, error = FALSE, irarg = -1, tempint, n_files = 0;
+	GMT_LONG	navg;	/* navg by navg pixels are averaged if navg > 1; else if navg == 1 do nothing */
+	GMT_LONG	iout, jout, iinstart, iinstop, jinstart, jinstop, k, kk, ion, jin, jj, iin, ii, kstart;
+	GMT_LONG nm;
+	GMT_LONG ij;
 	double	west, east, south, north, toplat, botlat, dx, rnavgsq, csum, dsum, left, bottom;
 	struct GRD_HEADER h;
 	struct GMT_IMG_COORD imgcoord;
@@ -211,7 +213,7 @@ int main (int argc, char **argv)
 		fprintf(stderr,"usage:  img2mercgrd <world_image_filename> %s -G<grdfile> -T<type>\n", GMT_Rgeo_OPT);
 		fprintf(stderr,"\t\t[-C] [-D[<minlat>/<maxlat>]] [-N<navg>] [-S<scale>] [-V] [-W<maxlon>] [-m<minutes>]\n\n");
 
-		if (GMT_give_synopsis_and_exit) exit (-1);
+		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 		
 		fprintf (stderr, "\tREQUIRED ARGUMENTS:\n");
 		fprintf(stderr,"\t<world_image_filename> gives location of img file.\n");
@@ -234,7 +236,7 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t-D[<minlon>/<maxlon>] input img file bottom and top latitudes. [%.3f/%.3f]\n", GMT_IMG_MINLAT, GMT_IMG_MAXLAT);
 		fprintf (stderr, "\t   If no latitudes are given it is taken to mean %.3f/%.3f\n", GMT_IMG_MINLAT_80, GMT_IMG_MAXLAT_80);
 		fprintf (stderr, "\t-W<maxlon> input img file runs from 0 to <maxlon> longitude. [360.0]\n");
-		exit(-1);
+		exit(EXIT_FAILURE);
 	}
 
 	if (irarg < 0 || west >= east || south >= north) {
@@ -279,7 +281,7 @@ int main (int argc, char **argv)
 		error++;
 	}
 	else if (Ctrl->N.value && (imgcoord.nx360%Ctrl->N.value != 0 || imgcoord.nyrow%Ctrl->N.value != 0) ) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR:  Bad choice of navg in -N.  Must divide %d and %d\n", GMT_program,
+		fprintf (stderr, "%s: GMT SYNTAX ERROR:  Bad choice of navg in -N.  Must divide %ld and %ld\n", GMT_program,
 			imgcoord.nx360, imgcoord.nyrow);
 		error++;
 	}
@@ -300,7 +302,7 @@ int main (int argc, char **argv)
 	botlat = GMT_img_ypix_to_lat ( (double)imgcoord.nyrow, &imgcoord);
 	dx = 1.0 / ( (double)imgcoord.nx360 / 360.0);
 	if (gmtdefs.verbose)
-		fprintf (stderr, "%s expects %s to be %d by %d pixels spanning 0/%5.1f/%.8g/%.8g\n",
+		fprintf (stderr, "%s expects %s to be %ld by %ld pixels spanning 0/%5.1f/%.8g/%.8g\n",
 			GMT_program, infile, imgcoord.nxcol, imgcoord.nyrow, dx*imgcoord.nxcol, botlat, toplat);
 
 	if (toplat < north) {
@@ -388,7 +390,8 @@ int main (int argc, char **argv)
 
 	row = (short int *) GMT_memory (VNULL, (size_t)(navg * imgcoord.nxcol), sizeof (short int), GMT_program);
 	ix = (int *) GMT_memory (VNULL, (size_t)(navgsq * h.nx), sizeof (int), GMT_program);
-	a = (float *) GMT_memory (VNULL, (size_t)(h.nx * h.ny), sizeof (float), GMT_program);
+	nm = GMT_get_nm (h.nx, h.ny);
+	a = (float *) GMT_memory (VNULL, nm, sizeof (float), GMT_program);
 
 	/* Load ix with the index to the correct column, for each output desired.  This helps for Greenwich, 
 		also faster averaging of the file, etc.  Note for averaging each n by n block is looped in turn. */
@@ -416,7 +419,7 @@ int main (int argc, char **argv)
 
 	/* Now before beginning data loop, fseek if needed.  */
 	if (jinstart > 0 && jinstart < imgcoord.nyrow) {
-		fseek (fp, 2 * imgcoord.nxcol * jinstart, SEEK_SET);
+		fseek (fp, (long)(2 * imgcoord.nxcol * jinstart), SEEK_SET);
 	}
 	
 	/* Now loop over output points, reading and handling data as needed */
@@ -430,7 +433,7 @@ int main (int argc, char **argv)
 		}
 		else {
 			if ( (fread((void *)row, sizeof (short int), (size_t)(navg * imgcoord.nxcol), fp) ) != (size_t)(navg * imgcoord.nxcol) ) {
-				fprintf(stderr,"%s:  ERROR:  Read failure at jin = %d.\n", GMT_program, jin);
+				fprintf(stderr,"%s:  ERROR:  Read failure at jin = %ld.\n", GMT_program, jin);
 				exit (EXIT_FAILURE);
 			}
 
@@ -502,7 +505,7 @@ int main (int argc, char **argv)
 	
 	Free_Img2mercgrd_Ctrl (Ctrl);	/* Deallocate control structure */
 
-        GMT_end (argc, argv);
+	GMT_end (argc, argv);
 
 	exit (EXIT_SUCCESS);
 }
@@ -510,7 +513,7 @@ int main (int argc, char **argv)
 void *New_Img2mercgrd_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct IMG2MERCGRD_CTRL *C;
 	
-	C = (struct IMG2MERCGRD_CTRL *) GMT_memory (VNULL, 1, sizeof (struct IMG2MERCGRD_CTRL), "New_Img2mercgrd_Ctrl");
+	C = (struct IMG2MERCGRD_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct IMG2MERCGRD_CTRL), "New_Img2mercgrd_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	C->D.min = GMT_IMG_MINLAT;
