@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    $Id: pspolar.c,v 1.26 2007/09/26 16:22:47 guru Exp $ 
+ *    $Id: pspolar.c,v 1.32 2009/07/11 03:16:31 guru Exp $ 
  *
- *    Copyright (c) 1996-2007 by G. Patau
+ *    Copyright (c) 1996-2009 by G. Patau
  *    Distributed under the GNU Public Licence
  *    See README file for copying and redistribution conditions.
  *--------------------------------------------------------------------*/
@@ -36,11 +36,11 @@
 
 #define POINTSIZE 0.005
 
-int main (int argc,char **argv)
+int main (int argc, char **argv)
 {
-    int     i, symbol = 0, n, n_files = 0, fno;
-    int     n_args;
-    int     form_s = 0, justify_s = 5;
+    GMT_LONG     i, symbol = 0, n, n_files = 0, fno;
+    GMT_LONG     n_args;
+    GMT_LONG     form_s = 0, justify_s = 5;
     
     BOOLEAN error = FALSE, nofile = TRUE;
     BOOLEAN done, greenwich, label_s = FALSE;
@@ -65,7 +65,7 @@ int main (int argc,char **argv)
     double si, co;
 
     char line[BUFSIZ], symbol_type, col[4][GMT_TEXT_LEN];
-    char pol;
+    char pol, *not_used = NULL;
     char txt_a[GMT_TEXT_LEN],txt_b[GMT_TEXT_LEN],txt_c[GMT_TEXT_LEN], txt_d[GMT_TEXT_LEN];
     char stacode[GMT_TEXT_LEN];
     
@@ -218,7 +218,7 @@ int main (int argc,char **argv)
                          if(strchr(argv[i], 'V')) {
                              vecS = TRUE;
                              strcpy(txt_a,strchr(argv[i], 'V'));
-                             if(strncmp(txt_a,"VG",2) == 0 || strncmp(txt_a,"VL",2) == 0 || strlen(txt_a) == 1) {
+                             if(strncmp(txt_a,"VG",(size_t)2) == 0 || strncmp(txt_a,"VL",(size_t)2) == 0 || strlen(txt_a) == 1) {
                                  v_width = 0.03; h_length = 0.12; h_width = 0.1; shape = gmtdefs.vector_shape;
                                  if (!gmtdefs.measure_unit) {
                                      v_width = 0.075; h_length = 0.3; h_width = 0.25; shape = gmtdefs.vector_shape;
@@ -250,7 +250,7 @@ int main (int argc,char **argv)
                 case 'T':       /* Information about label printing */
                         label_s = TRUE;
                         if (strlen(argv[i]) > 2) {
-                             sscanf (&argv[i][2], "%lf/%d/%d/%lf/", &angle_s,
+                             sscanf (&argv[i][2], "%lf/%ld/%ld/%lf/", &angle_s,
                                      &form_s, &justify_s, &fontsize_s);
                         }
                         break;
@@ -404,7 +404,7 @@ int main (int argc,char **argv)
             sprintf (line, "File: %s", argv[fno]);
             ps_comment (line);
         }
-        if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) GMT_fgets (line, 512, fp);
+        if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) not_used = GMT_fgets (line, 512, fp);
         
         GMT_world_map = TRUE;
         
