@@ -1,7 +1,7 @@
 /*-------------------------------------------------------------------------
- *	$Id: mgd77.h,v 1.82 2007/06/14 20:19:28 guru Exp $
+ *	$Id: mgd77.h,v 1.117 2009/07/07 08:17:21 guru Exp $
  * 
- *    Copyright (c) 2005-2007 by P. Wessel
+ *    Copyright (c) 2005-2009 by P. Wessel
  *    See README file for copying and redistribution conditions.
  *
  *  File:	mgd77.h
@@ -9,7 +9,7 @@
  *  Include file for programs that plan to read/write MGD77[+] files
  *
  *  Authors:    Paul Wessel, Primary Investigator, SOEST, U. of Hawaii
- *		Michael Chandler, Master's Candidate, SOEST, U. of Hawaii
+ *				Michael Chandler, Ph.D. Student, SOEST, U. of Hawaii
  *		
  *  Version:	1.3
  *  Revised:	15-MAR-2006
@@ -33,7 +33,8 @@
 #define MGD77_N_DATA_EXTENDED	28		/* The 27 plus time */
 #define MGD77_N_NUMBER_FIELDS	24		/* Original 24 numerical data columns in MGD77 */
 #define MGD77_N_STRING_FIELDS	3		/* Original 3 text data columns in MGD77 */
-#define MGD77_N_HEADER_ITEMS	66		/* Number of individual header items in the MGD77 header */
+#define MGD77_N_HEADER_ITEMS	72		/* Number of individual header items in the MGD77 header */
+#define MGD77_N_MAG_RF		18		/* Number of different Mag ref fields so far in MGD77 docs */
 /* The 28 MGD77 standard types (27 original + 1 conglomerate (time)) */
 #define MGD77_RECTYPE		0
 #define MGD77_TZ		1
@@ -103,12 +104,20 @@
 #define MGD77_IGF_1967		3
 #define MGD77_IGF_1980		4
 
+#define MGD77_IGRF_LAST_ID	20	/* IGRF fields IDs in MGD77 are 3,4,11-MGD77_IGRF_LAST_ID.  Update when more fields are added */
+
+#define TWT_PDR_WRAP	10.0						/* The 10 second PDR wrap-around we see in SIO cruises */
+#define TWT_PDR_WRAP_TRIGGER	0.5 * TWT_PDR_WRAP	/* Any jump in TWT that exceeds this triggers a wrap */
+
 #define GMT_IMG_MINLAT -72.0059773539
 #define GMT_IMG_MAXLAT +72.0059773539
 
 #define MGD77_BAD_HEADER_RECNO		-1
 #define MGD77_BAD_HEADER_ITEM		-2
 
+#define MGD77_ORIG			0
+#define MGD77_REVISED			1
+
 /* Return error numbers */
 
 #define MGD77_NO_ERROR			0
@@ -134,6 +143,16 @@
 #define MGD77_BAD_ARG			20
 #define MGD77_BAD_IGRFDATE		21
 
+/* For mgd77+ files */
+
+#define NCPOS_TIME	0
+#define NCPOS_LAT	3
+#define NCPOS_LON	4
+#define NCPOS_TWT	6
+#define NCPOS_MTF1	10
+#define NCPOS_GOBS	16
+#define NCPOS_EOT	21
+
 /* We will use bit flags to keep track of which data column we are referring to.
  * field 0 is rightmost bit (1), field 1 is the next bit (2), field 2 is 4 and
  * so on for powers of 2.  We add these bit powers together to get the bit patterns
@@ -165,7 +184,6 @@
 #define MGD77_BIT		6
 #define MGD77_NEQ		8
 
-#define N_AUX	15		/* Number of auxilliary columns in mgd77list */
 
 typedef char byte;	/* Used to indicate 1-byte long integer */
 typedef char* Text;	/* Used to indicate character strings */
@@ -181,6 +199,12 @@ typedef char* Text;	/* Used to indicate character strings */
 #define MGD77_COL_UNIT_LEN	64
 #define MGD77_COL_COMMENT_LEN	128
 
+#define MGD77_COL_ADJ_TWT	1	/* Undo twt PDR wraps given PDR_wrap value */
+#define MGD77_COL_ADJ_DEPTH	2	/* Compute Carter depth from twt */
+#define MGD77_COL_ADJ_MAG	3	/* Compute mag from mtf1 - igrf */
+#define MGD77_COL_ADJ_FAA	4	/* Compute faa from gobs - igf */
+#define MGD77_COL_ADJ_FAA_EOT	5	/* Compute faa from gobs - igf + eot */
+
 struct MGD77_COLINFO {
 	char *abbrev;		/* Short name that identifies this column */
 	char *name;		/* Longer, descriptive name for column */
@@ -195,6 +219,7 @@ struct MGD77_COLINFO {
 	nc_type type;		/* Type of representation of this data in the netCDF file (NC_SHORT, NC_INT, NC_BYTE, etc) */
 	char text;		/* length if this is a text string, else 0 */
 	int var_id;		/* netCDF variable ID */
+	int adjust;		/* Column needs some sort of adjustment before data is returned [0 means as is] */
 	BOOLEAN constant;	/* TRUE if column is constant and only 1 row is/should be stored */
 	BOOLEAN present;	/* TRUE if column is present in the file (NaN or otherwise) */
 };
@@ -212,21 +237,94 @@ struct MGD77_META {	/* Information about a cruise as derived from navigation dat
 	int Departure[3];	/* yyyy, mm, dd of departure */
 	int Arrival[3];		/* yyyy, mm, dd of arrival */
 	signed char ten_box[20][38];	/* Set to 1 for each box visited */
+	double G1980_1930;	/* Average difference between 1980 and 1930 gravity reference field for this cruise */
 };
 
 struct MGD77_HEADER {	
-	struct MGD77_HEADER_PARAMS *mgd77;		/* See MGD-77 Documentation from NGDC for details */
+	struct MGD77_HEADER_PARAMS *mgd77[2];		/* See MGD-77 Documentation from NGDC for details; [0] is ORIG, [1] is REVISED */
 	struct MGD77_META meta;				/* Holds some meta-data derived directly from data records */
 	char *author;					/* Name of author of last creation/modification */
 	char *history;					/* History of creation/modifications */
 	char *E77;					/* Statement of E77 information encoded */
-	int n_records;					/* Number of MGD77 data records found */
+	GMT_LONG n_records;					/* Number of MGD77 data records found */
 	int n_fields;					/* Number of columns returned */
 	int errors[3];					/* Number of total errors, (warnings, errors) found when reading this header */
 	BOOLEAN no_time;				/* TRUE for those few cruises that have no time values */
+	double PDR_wrap;				/* Non-zero if we must undo PDR wrapping */
 	struct MGD77_DATA_INFO info[MGD77_N_SETS];	/* Info regarding [0] standard MGD77 columns and [1] any extra columns (max 32 each) */
 };
 
+#ifdef USE_CM4
+typedef int logical;
+struct MGD77_CM4 {	/* For use with cm4field.c and initialized by MGD77_CM4_init () */
+	char *path[3];		/* Paths to the three coefficient files */
+	int unit[3];		/* IDs of logical units */
+	logical load[3];	/* TRUE if the file has been read into memory */
+	logical index[2];	/* Index acquisition flags/ TRUE from file, FALSE from argument */
+	logical gmut;		/* Magnetic dipole universal time (MUT) acquisition flag: TRUE, compute from UT, FALSE, from arg */
+	logical cord;		/* TRUE for geodetic, FALSE for geocentric */
+	logical pred[6];	/* TRUE to compute, FALSE not: 1 = main field, 2 magnetispheric, 3 ionospheric, etc */
+	logical curr;		/* Model J current field prediction flag: TRUE, compute, FALSE not */
+	logical coef;		/* Model coefficient generation flag: TRUE, compute, FALSE not */
+	int nhmf[2];		/* Maximum main field spherical harmonic degree (0 for main field 1, 1 for main field 2) */
+	int nlmf[2];		/* Minimum main field spherical harmonic degree (0 for main field 1, 1 for main field 2) */
+	double mut;		/* Magnetic dipole universal time (hours 0-24), computed from UT and returned as gmut = TRUE */
+	double alt;		/* Altitude [0] */
+	double dst;		/* Linearly interpolated hourly Dst magnetic index, returned from file as index[0] = TRUE */
+	double f107;		/* Linearly interpolated 3-monthly means of absolute F10.7 solar radiation flux value, returned */
+	double bmdl[3][7];	/* Array storing computed B field vectors from various sources (nT) */
+	double jmdl[3][4];	/* Array storing computed J field vectors from certain external sources */
+	double *gmdl;		/* Array storing coefficients from various sources */
+	int perr;		/* Error message print flag: 0 do not print */
+	int oerr;		/* Unit to print to */
+	int cerr;		/* Error return code (0 = normal, 1-49 warning, > 50 fatal */
+};
+#endif
+
+/* We may want to output columns that themselves are not stored in the MGD77[+] files but
+ * rather are computed based on data that are stored in the file.  We consider such information
+ * as AUXILLARY columns and insert them between the observed columns when needed.  The following
+ * structures are used to facilitate this process. */
+
+#ifdef USE_CM4
+#define N_MGD77_AUX	16		/* Number of auxilliary derived columns for MGD77 data, including optional CM4 */
+#else
+#define N_MGD77_AUX	15		/* Number of auxilliary derived columns for MGD77 data */
+#endif
+#define N_GENERIC_AUX	3		/* Number of auxilliary derived columns for general files (dist, azim, vel) */
+
+#define MGD77_AUX_DS	0
+#define MGD77_AUX_AZ	1
+#define MGD77_AUX_SP	2
+#define MGD77_AUX_YR	3
+#define MGD77_AUX_MO	4
+#define MGD77_AUX_DY	5
+#define MGD77_AUX_HR	6
+#define MGD77_AUX_MI	7
+#define MGD77_AUX_SC	8
+#define MGD77_AUX_WT	9
+#define MGD77_AUX_RT	10
+#define MGD77_AUX_MG	11
+#define MGD77_AUX_CT	12
+#define MGD77_AUX_GR	13
+#define MGD77_AUX_ID	14
+#ifdef USE_CM4
+#define MGD77_AUX_CM	15
+#endif
+
+struct MGD77_AUXLIST {
+	char name[MGD77_COL_ABBREV_LEN];
+	GMT_LONG type;
+	GMT_LONG text;
+	GMT_LONG requested;
+	char header[GMT_TEXT_LEN];
+};
+
+struct MGD77_AUX_INFO {
+	GMT_LONG type;
+	GMT_LONG text;
+	GMT_LONG pos;
+};
 
 /* The data records in the MGD77 file consist of records that are 120 characters.
  * This information can be read and stored internally in the structure MGD77_DATA_RECORD.
@@ -239,10 +337,11 @@ struct MGD77_DATA_RECORD {	/* See MGD77 Documentation from NGDC for details */
 	double time;				/* Time using current GMT absolute time conventions (J2000 UTC) */
 	char word[MGD77_N_STRING_FIELDS][10];	/* The 3 text strings in MGD77 records */
 	unsigned int bit_pattern;		/* Bit pattern indicating which of the 27 fields are present in current record */
+	BOOLEAN keep_nav;				/* Set to false when navigation is bad */
 };
 
 struct MGD77_DATASET {	/* Info for an entire MGD77+ data set */
-	int n_fields;				/* Number of active colums in the values table */
+	int n_fields;				/* Number of active columns in the values table */
 	int errors;				/* Number of errors encountered when writing this data */
 	struct MGD77_HEADER H;			/* The file's header information */
 	void *values[MGD77_MAX_COLS];		/* 2-D table of necessary number of columns and rows (mix of double and char pointers) */
@@ -303,12 +402,15 @@ struct MGD77_CONTROL {
 	int verbose_dest;				/* 1 = to stdout, 2 = to stderr */
 	int nc_id;					/* netCDF ID for current open file (MGD77+ only) */
 	int nc_recid;					/* netCDF ID for dimension of records (time) */
-	int rec_no;					/* Current record to read/write for record-based i/o */
+	GMT_LONG rec_no;					/* Current record to read/write for record-based i/o */
 	int format;					/* 0 if any file format, 1 if MGD77, and 2 if netCDF, 3 if ascii table */
 	/* Format-related issues */
 	int time_format;				/* Either GMT_IS_ABSTIME or GMT_IS_RELTIME */
+	struct GMT_TIME_SYSTEM utime;			/* All the information about the Unix time system */
+	BOOLEAN adjust_time;				/* TRUE if GMT time-system is NOT unix */
 	/* Data use information */
 	BOOLEAN original;				/* TRUE means we want original not revised header attributes */
+	BOOLEAN revised;				/* TRUE means we are working on a MGD77+ file with revised header attributes */
 	BOOLEAN Want_Header_Item[MGD77_N_HEADER_ITEMS];	/* TRUE means print this header item if dump is selected */
 	BOOLEAN use_flags[MGD77_N_SETS];		/* TRUE means programs will use error bitflags (if present) when returning data */
 	BOOLEAN use_corrections[MGD77_N_SETS];		/* TRUE means we will apply correction factors (if present) when reading data */
@@ -317,7 +419,7 @@ struct MGD77_CONTROL {
 	int n_constraints;				/* Number of constraints specified */
 	int n_exact;					/* Number of exact columns to match */
 	int n_bit_tests;				/* Number of bit tests to match */
-	int no_checking;				/* TRUE if there are no constraints, extact-tests, or bit-tests to pass */
+	int no_checking;				/* TRUE if there are no constraints, exact-tests, or bit-tests to pass */
 	struct MGD77_CONSTRAINT Constraint[MGD77_MAX_COLS];		/* List of constraints, if any */
 	char desired_column[MGD77_MAX_COLS][MGD77_COL_ABBREV_LEN];	/* List of desired column names in final output order */
 	struct MGD77_PAIR Exact[MGD77_MAX_COLS];	/* List of column names whose values must be !NaN to be output, if any */
@@ -337,7 +439,7 @@ struct MGD77_CARTER {
 	short int carter_correction[N_CARTER_CORRECTIONS];
 };
 
-/* Structures for emphemeral corrections */
+/* Structures for ephemeral corrections */
 
 struct MGD77_CORRECTION {	/* Holds parameters for one term of a correction for one kind of observation */
 	int id;			/* The id - entry to give us the data column to use*/
@@ -357,7 +459,9 @@ struct MGD77_CORRTABLE {
 
 extern void MGD77_Init (struct MGD77_CONTROL *F);						/* Initialize the MGD77 machinery */
 extern void MGD77_Reset (struct MGD77_CONTROL *F);									/* Reset after finishing a file */
+extern void MGD77_end (struct MGD77_CONTROL *F);				/* Free up MGD77-related variables */
 extern int MGD77_Path_Expand (struct MGD77_CONTROL *F, char **argv, int argc, char ***list);				/* Returns the full list of IDs */
+extern void MGD77_Path_Free (int n, char **list);	/* Free the list of IDs */
 extern void MGD77_Cruise_Explain (void);										/* Explains how to specify IDs */
 extern int MGD77_Get_Path (char *track_path, char *track, struct MGD77_CONTROL *F);					/* Returns full path to cruise */
 extern int MGD77_Open_File (char *leg, struct MGD77_CONTROL *F, int rw);						/* Opens a MGD77[+] file */
@@ -366,6 +470,7 @@ extern int MGD77_Read_File (char *file, struct MGD77_CONTROL *F, struct MGD77_DA
 extern int MGD77_Write_File (char *file, struct MGD77_CONTROL *F, struct MGD77_DATASET *S);				/* Write entire file (all columns) */
 extern int MGD77_Read_Header_Record (char *file, struct MGD77_CONTROL *F, struct MGD77_HEADER *H);			/* Read the header record */
 extern int MGD77_Write_Header_Record (char *file, struct MGD77_CONTROL *F, struct MGD77_HEADER *H);			/* Write the header record */
+extern int MGD77_Free_Header_Record (struct MGD77_CONTROL *F, struct MGD77_HEADER *H);					/* Frees up header memory */
 extern int MGD77_Read_Data (char *file, struct MGD77_CONTROL *F, struct MGD77_DATASET *S);				/* Allocate & Read all data (selected columns only); Header already read */
 extern int MGD77_Write_Data (char *file, struct MGD77_CONTROL *F, struct MGD77_DATASET *S);				/* Write all data (all columns); Header already written */
 extern int MGD77_Read_Data_Record (struct MGD77_CONTROL *F, struct MGD77_HEADER *H, double dvals[], char *tvals[]);	/* Read a single data record (selected columns only) */
@@ -378,8 +483,8 @@ extern void MGD77_List_Header_Items (struct MGD77_CONTROL *F);
 extern int MGD77_Select_Header_Item (struct MGD77_CONTROL *F, char *item);
 extern int MGD77_Get_Set (char *abbrev);										/* Returns 0 if abbrev is in the MGD77 set, else 1 */
 extern void MGD77_Fatal_Error (int error);										/* Print message for this error and exit */
-extern BOOLEAN MGD77_Pass_Record (struct MGD77_CONTROL *F, struct MGD77_DATASET *S, int rec);				/* Tests if a record passes all specified logical & exact tests */
-extern void MGD77_Apply_Bitflags (struct MGD77_CONTROL *F, struct MGD77_DATASET *S, int rec, BOOLEAN apply_bits[]);	/* Replaces values whose flags are ON with NaNs */
+extern BOOLEAN MGD77_Pass_Record (struct MGD77_CONTROL *F, struct MGD77_DATASET *S, GMT_LONG rec);				/* Tests if a record passes all specified logical & exact tests */
+extern void MGD77_Apply_Bitflags (struct MGD77_CONTROL *F, struct MGD77_DATASET *S, GMT_LONG rec, BOOLEAN apply_bits[]);	/* Replaces values whose flags are ON with NaNs */
 extern void MGD77_Set_Unit (char *dist, double *scale, int way);							/* Convert appended distance unit to a numerical scale to give meters */
 extern void MGD77_nc_status (int status);										/* Checks for netCDF errors and aborts with error message */
 extern void MGD77_Process_Ignore (char code, char *format);								/* Process the ignre-format option */
@@ -389,7 +494,7 @@ extern void MGD77_Prep_Header_cdf (struct MGD77_CONTROL *F, struct MGD77_DATASET
 extern void MGD77_Dump_Header_Params (struct MGD77_CONTROL *F, struct MGD77_HEADER_PARAMS *P);				/* Dump of header items, one per line */
 extern void MGD77_Verify_Header (struct MGD77_CONTROL *F, struct MGD77_HEADER *H, FILE *ufp);				/* Verify content of header per MGD77 docs */
 extern void MGD77_Verify_Prep (struct MGD77_CONTROL *F, struct MGD77_DATASET *D);
-extern void MGD77_Verify_Prep_m77 (struct MGD77_CONTROL *F, struct MGD77_META *C, struct MGD77_DATA_RECORD *D, int nrec);
+extern void MGD77_Verify_Prep_m77 (struct MGD77_CONTROL *F, struct MGD77_META *C, struct MGD77_DATA_RECORD *D, GMT_LONG nrec);
 extern int MGD77_Remove_E77 (struct MGD77_CONTROL *F);
 
 /* Secondary user functions */
@@ -401,10 +506,12 @@ extern int MGD77_Write_Data_Record_m77 (struct MGD77_CONTROL *F, struct MGD77_DA
 
 /* These are only for developers */
 
-extern int MGD77_dbl_are_constant (double x[], int n, double limits[]);
-extern int MGD77_txt_are_constant (char *txt, int n, int width);
-extern int MGD77_do_scale_offset_before_write (double new[], const double x[], int n, double scale, double offset, int type);
+extern BOOLEAN MGD77_dbl_are_constant (double x[], GMT_LONG n, double limits[]);
+extern BOOLEAN MGD77_txt_are_constant (char *txt, GMT_LONG n, int width);
+extern int MGD77_do_scale_offset_before_write (double new[], const double x[], GMT_LONG n, double scale, double offset, int type);
 extern void MGD77_select_high_resolution ();
+extern void MGD77_free_plain_mgd77 (struct MGD77_HEADER *H);
+extern int MGD77_Match_List (char *word, int n_fields, char **list);
 
 /* User functions for direct use of Carter corrections */
 
@@ -419,13 +526,28 @@ extern double MGD77_carter_correction (double lon, double lat, double twt_in_mse
 extern int MGD77_igrf10syn (int isv, double date, int itype, double alt, double lon, double lat, double *out);
 extern double MGD77_Theoretical_Gravity (double lon, double lat, int version);
 extern void MGD77_IGF_text (FILE *fp, int version);
+extern double MGD77_Recalc_Mag_Anomaly_IGRF (struct MGD77_CONTROL *F, double time, double lon, double lat, double obs, BOOLEAN calc_date);
+extern double MGD77_time_to_fyear (struct MGD77_CONTROL *F, double time);
+extern double MGD77_cal_to_fyear (struct GMT_gcal *cal);
+extern BOOLEAN MGD77_fake_times (struct MGD77_CONTROL *F, struct MGD77_HEADER *H, double *lon, double *lat, double *times, GMT_LONG nrec);
+extern double MGD77_utime2time (struct MGD77_CONTROL *F, double unix_time);
+extern double MGD77_time2utime (struct MGD77_CONTROL *F, double gmt_time);
+extern double MGD77_rdc2dt (struct MGD77_CONTROL *F, GMT_cal_rd rd, double secs);
+extern void MGD77_dt2rdc (struct MGD77_CONTROL *F, double t, GMT_cal_rd *rd, double *s);
+extern void MGD77_gcal_from_dt (struct MGD77_CONTROL *F, double t, struct GMT_gcal *cal);
+
+#ifdef USE_CM4 
+extern double MGD77_Calc_CM4 (struct MGD77_CONTROL *F, double time, double lon, double lat, BOOLEAN calc_date, struct MGD77_CM4 *CM4);
+extern double MGD77_Recalc_Mag_Anomaly_CM4 (struct MGD77_CONTROL *F, double time, double lon, double lat, double obs, BOOLEAN calc_date, struct MGD77_CM4 *CM4);
+extern void MGD77_CM4_end (struct MGD77_CM4 *CM4);
+#endif
 
 /* These are called indirectly but remain accessible for specialist programs */
 
 extern int MGD77_carter_init (struct MGD77_CARTER *C);
 extern int MGD77_carter_get_bin (double lon, double lat, int *bin);
 extern int MGD77_carter_get_zone (int bin, struct MGD77_CARTER *C, int *zone);
-extern double *MGD77_Distances (double x[], double y[], int n, int dist_flag);
+extern double *MGD77_Distances (double x[], double y[], GMT_LONG n, int dist_flag);
 
 /* Global variables used by MGD77 programs */
 
@@ -436,10 +558,15 @@ extern BOOLEAN MGD77_format_allowed[MGD77_N_FORMATS];	/* By default we allow ope
 extern double MGD77_Epoch_zero;
 extern int MGD77_pos[MGD77_N_DATA_EXTENDED];
 
-void MGD77_Parse_Corrtable (struct MGD77_CONTROL *F, char *tablefile, char **cruises, int n_cruises, struct MGD77_CORRTABLE ***CORR);
+int MGD77_Scan_Corrtable (char *tablefile, char **cruises, int n_cruises, int n_fields, char **field_names, char ***item_names, int mode);
+void MGD77_Parse_Corrtable (char *tablefile, char **cruises, int n_cruises, int n_fields, char **field_names, int mode, struct MGD77_CORRTABLE ***CORR);
 void MGD77_Init_Correction (struct MGD77_CORRTABLE *CORR, double **value);
-double MGD77_Correction (struct MGD77_CORRECTION *C, double **value, double *aux, int rec);
+double MGD77_Correction (struct MGD77_CORRECTION *C, double **value, double *aux, GMT_LONG rec);
+double MGD77_Correction_Rec (struct MGD77_CORRECTION *C, double *value, double *aux);
+void MGD77_Free_Correction (struct MGD77_CORRTABLE **CORR, int n);
 
 #include "mgd77_functions.h"	/* These were created by mgd77netcdfhelper.sh */
+#include "cm4_functions.h"
+extern void MGD77_CM4_init (struct MGD77_CONTROL *F, struct MGD77_CM4 *CM4);
 
 #endif	/* _MGD77_H */
