@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------
- *	$Id: mgd77manage.c,v 1.111 2009/01/21 02:01:50 guru Exp $
+ *	$Id: mgd77manage.c,v 1.130 2009/07/11 03:16:31 guru Exp $
  *
  *    Copyright (c) 2005-2009 by P. Wessel
  * mgd77manage is used to (1) remove data columns from mgd77+ files
@@ -21,18 +21,23 @@
 #include "mgd77.h"
 #include "mgd77_e77.h"	/* E77 Header Errata Codes */
 
-#define N_PAR		6
+#define N_PAR		7
 #define COL_SCALE	0
 #define COL_OFFSET	1
 #define IMG_SPACING	2
 #define IMG_SCALE	3
 #define IMG_MODE	4
-#define COL_TYPE	5
+#define IMG_LAT		5
+#define COL_TYPE	6
 
 #define ADD_IGRF	2
 #define ADD_CARTER	3
 #define ADD_GRAV	4
 #define ADD_RMAG	5
+#ifdef USE_CM4
+#define ADD_CM4		6
+#define ADD_RMAG4	7
+#endif
 
 #define N_E77_MODES	5
 #define E77_HEADER_MODE	0
@@ -47,12 +52,12 @@ int main (int argc, char **argv)
 {
 	int var_id, n_dims = 0, dims[2];
 	
-	int i, j, k = 0, ii, jj, argno, n_cruises = 0, n_paths = 0, mx = 0, my, pos, column, result;
+	int i, j, k = 0, ii, jj, argno, n_cruises = 0, n_paths = 0, mx = 0, my, column, result;
 	int width, n_delete = 0, GF_version = MGD77_NOT_SET, n_expected_fields, n_fields = 0;
-	int kind = GMT_IS_FLOAT, MTF_col = 1, set, dist_flag = 2, n_bad, e77_mode[N_E77_MODES];
+	int kind = GMT_IS_FLOAT, MTF_col = 1, set, dist_flag = 2, n_bad;
 	int interpolant = BCR_BICUBIC, check;
 	
-	GMT_LONG ij, n_sampled = 0, n_changed = 0, n = 0;
+	GMT_LONG n_sampled = 0, n_changed = 0, n = 0, pos;
 	
 	size_t n_alloc = GMT_CHUNK, start[2] = {0, 0}, count[2] = {0, 0};
 	
@@ -61,16 +66,15 @@ int main (int argc, char **argv)
 	nc_type c_nc_type;
 	
 	BOOLEAN got_c = FALSE, got_a = FALSE, got_d = FALSE, got_g = FALSE, got_i = FALSE, got_n = FALSE, got_t = FALSE;
-	BOOLEAN error = FALSE, replace = FALSE, delete = FALSE, interpolate = FALSE, transform, verified;
-	BOOLEAN strings = FALSE, track_decode, got_grid, got_table, two_cols = FALSE, constant, ok_to_read = TRUE, got_e = FALSE;
+	BOOLEAN error = FALSE, replace = FALSE, delete = FALSE, interpolate = FALSE, transform, verified, force = FALSE;
+	BOOLEAN strings = FALSE, got_grid, got_table, two_cols = FALSE, constant, ok_to_read = TRUE, got_e = FALSE;
+	BOOLEAN ignore_verify = FALSE, e77_skip_mode[N_E77_MODES];
 	
 	char line[BUFSIZ], file[BUFSIZ], *d_list = CNULL, p[BUFSIZ], history[BUFSIZ], c_size = 0, **list;
 	char c_abbrev[GMT_TEXT_LEN], c_units[GMT_TEXT_LEN], c_name[MGD77_COL_NAME_LEN], c_comment[MGD77_COL_COMMENT_LEN];
 	char no_char = '9', not_given[GMT_TEXT_LEN], word[BUFSIZ], **tmp_string = VNULL, *text = VNULL, d_unit[2];
 	signed char LEN = 0, OLDLEN = 0;
 	
-	short int *i2;
-	
 	float *f = VNULL;
 
 	double threshold = 1.0, i_dx = 0, i_dy = 0, x, y, match_value, single_val, dist_scale = 1.0;
@@ -81,7 +85,6 @@ int main (int argc, char **argv)
 	struct GRD_HEADER grd;
 	struct GMT_EDGEINFO edgeinfo;
 	struct GMT_BCR bcr;
-	struct GMT_gcal cal;
 	struct MGD77_CARTER Carter;
 	
 	FILE *fp, *fp_err;
@@ -92,6 +95,8 @@ int main (int argc, char **argv)
 	int got_default_answer (char *line, char *answer);
 
 	argc = GMT_begin (argc, argv);		/* Initialize GMT Machinery */
+	GMT_get_time_system ("unix", &(gmtdefs.time_system));						/* MGD77+ uses GMT's Unix time epoch */
+	GMT_init_time_system_structure (&(gmtdefs.time_system));
 	
 	GMT_boundcond_init (&edgeinfo);
 	parameters[COL_SCALE]   = 1.0;	/* Output column scaling */
@@ -99,11 +104,12 @@ int main (int argc, char **argv)
 	parameters[IMG_SPACING] = 0.0;	/* IMG data minute spacing */
 	parameters[IMG_SCALE]   = 1.0;	/* IMG data scaling */
 	parameters[IMG_MODE]    = 0.0;	/* IMG data mode */
+	parameters[IMG_LAT ]    = 0.0;	/* IMG lat not set */
 	parameters[COL_TYPE]    = 0;	/* netCDF type */
 	GMT_pad[0] = GMT_pad[1] = GMT_pad[2] = GMT_pad[3] = 2;
 	d_unit[0] = d_unit[1] = '\0';
-	memset ((void *)e77_mode, 0, (size_t)(N_E77_MODES * sizeof (int)));
-	/* Default e77_mode will apply header and fix corrections if prefix is Y and set all data bits */
+	memset ((void *)e77_skip_mode, 0, (size_t)(N_E77_MODES * sizeof (BOOLEAN)));
+	/* Default e77_skip_mode will apply header and fix corrections if prefix is Y and set all data bits */
 	
 	MGD77_Init (&In);			/* Initialize MGD77 Machinery */
 
@@ -138,24 +144,26 @@ int main (int argc, char **argv)
 							got_d = TRUE;
 							error += decode_A_options (0, &argv[i][k+1], file, parameters);
 							break;
+						case 'E':	/* Plain E77 error flag file from mgd77sniffer */
+							ignore_verify = TRUE;	/* Process raw e77 files that have not been verified */
 						case 'e':	/* Plain E77 error flag file from mgd77sniffer */
 							got_e = TRUE;
 							while (argv[i][++k]) {
 								switch (argv[i][k]) {
 									case 'h':	/* Ignore all header recommendations regardless of Y/N prefix */
-										e77_mode[E77_HEADER_MODE] = -1;
+										e77_skip_mode[E77_HEADER_MODE] = TRUE;
 										break;
 									case 'f':	/* Ignore all systematic trend recommendations regardless of Y/N prefix */
-										e77_mode[E77_TREND_MODE] = -1;
+										e77_skip_mode[E77_TREND_MODE] = TRUE;
 										break;
 									case 'n':	/* Ignore all NAV flags */
-										e77_mode[E77_NAV_MODE] = -1;
+										e77_skip_mode[E77_NAV_MODE] = TRUE;
 										break;
 									case 'v':	/* Ignore all VALUE flags */
-										e77_mode[E77_VALUES_MODE] = -1;
+										e77_skip_mode[E77_VALUES_MODE] = TRUE;
 										break;
 									case 's':	/* Ignore all SLOPE flags */
-										e77_mode[E77_SLOPES_MODE] = -1;
+										e77_skip_mode[E77_SLOPES_MODE] = TRUE;
 										break;
 									default:
 										fprintf (stderr, "%s: ERROR: -Ae modifiers must be combination of hfnvs\n", GMT_program);
@@ -206,6 +214,9 @@ int main (int argc, char **argv)
 					no_char = argv[i][2];
 					break;
 
+				case 'F':	/* Force mode */
+					force = TRUE;
+
 				case 'I':	/* Column attribute information */
 					error += decode_I_options (&argv[i][2], c_abbrev, c_name, c_units, &c_size, c_comment, parameters);
 					break;
@@ -252,8 +263,8 @@ int main (int argc, char **argv)
 	
 	if (GMT_give_synopsis_and_exit || argc == 1) {	/* Display usage */
 		fprintf(stderr,"mgd77manage %s - Manage the content of MGD77+ files\n\n", MGD77_VERSION);
-		fprintf(stderr,"usage: mgd77manage <cruise(s)> [-A[+]a|c|d|D|e|g|i|n|t|T<info>] [-Cf|g|e] [-Dname1,name2,...]\n");
-		fprintf(stderr,"\t[-E<no_char>] [-I<abbrev>/<name>/<units>/<size>/<scale>/<offset>/\"comment\"]\n");
+		fprintf(stderr,"usage: mgd77manage <cruise(s)> [-A[+]a|c|d|D|e|E|g|i|n|t|T<info>] [-Cf|g|e] [-Dname1,name2,...]\n");
+		fprintf(stderr,"\t[-E<no_char>] [-F] [-I<abbrev>/<name>/<units>/<size>/<scale>/<offset>/\"comment\"]\n");
 		fprintf(stderr,"\t[-Ne|k|m|n[+|-]] [-Q[b|c|l|n][[/]<threshold>]] [-V] [%s]\n\n", GMT_bi_OPT);
          
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
@@ -267,8 +278,14 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t      with the same number of records as the MGD77 file.  Only one cruise can be set.\n");
 		fprintf (stderr, "\t      If filename is - we read from stdin.\n");
 		fprintf (stderr, "\t   c: Create a new column to be calculated from existing columns.  Add code:\n");
+#ifdef USE_CM4
+		fprintf (stderr, "\t        4 = CM4 field, m = IGRF total field, c = Carter correction, g = IGF (\"normal gravity\")\n");
+		fprintf (stderr, "\t        R = recomputed magnetic anomaly rmag = mtfx - CM4 total field\n");
+		fprintf (stderr, "\t        r = recomputed magnetic anomaly rmag = mtfx - IGRF total field\n");
+#else
 		fprintf (stderr, "\t        m = IGRF total field, c = Carter correction, g = IGF (\"normal gravity\")\n");
 		fprintf (stderr, "\t        r = recomputed magnetic anomaly rmag = mtfx - IGRF total field\n");
+#endif
 		fprintf (stderr, "\t        Append x for which mtfx field to use (1 or 2) [1]\n");
 		fprintf (stderr, "\t        For g, optionally append 1-4 to select the gravity formula to use:\n");
 		fprintf (stderr, "\t        1 = Heiskanen 1924, 2 = International 1930, 3 = IGF1967, 4 = IGF1980.\n");
@@ -286,12 +303,13 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t        n = Ignore data record bitflags pertaining to navigation (time, lon, lat).\n");
 		fprintf (stderr, "\t        v = Ignore data record bitflags pertaining to data values.\n");
 		fprintf (stderr, "\t        s = Ignore data record bitflags pertaining to data slopes (gradients).\n");
+		fprintf (stderr, "\t      Use -DE to ignore the verification status of the e77 file [Default requires verification to be Y]\n");
 		fprintf (stderr, "\t      NOTE:  Previous E77 information will be removed prior to processing this E77 information.\n");
 		fprintf (stderr, "\t   g: Sample a GMT grid along track. (also see -Q).\n");
 		fprintf (stderr, "\t      Append filename of the GMT grid.\n");
 		fprintf (stderr, "\t   i: Sample a Sandwell/Smith *.img Mercator grid along track (also see -Q).\n");
-		fprintf (stderr, "\t      Append /scale/mode/filename to set the grid's scale, mode, and filename:\n");
-		fprintf (stderr, "\t      Give scale (0.1 or 1) to multiply after read, and set mode as follows:\n");
+		fprintf (stderr, "\t      Give filename and append comma-separated scale, mode, and optionally max latitude [%g].\n", GMT_IMG_MAXLAT_80);
+		fprintf (stderr, "\t      The scale (0.1 or 1) is used to multiply after read; give mode as follows:\n");
 		fprintf (stderr, "\t        0 = img file w/ no constraint code, interpolate to get data at track.\n");
                 fprintf (stderr, "\t        1 = img file w/ constraints coded, interpolate to get data at track.\n");
                 fprintf (stderr, "\t        2 = img file w/ constraints coded, gets data only at constrained points, NaN elsewhere.\n");
@@ -312,6 +330,7 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t   The columns are removed before any data are added.  It is not a substitute for -A+.\n");
 		fprintf (stderr, "\t   However, sometimes the shape of new data demands the old to be deleted first (you will be told)\n");
 		fprintf (stderr, "\t-E Give character used to fill empty/missing string columns [9]\n");
+		fprintf (stderr, "\t-F Force mode.  This allows you to even replace the standard MGD77 columns [only extended columns can be changed]\n");
 		fprintf (stderr, "\t-I In addition to the file information above, you must also specify column information:\n");
 		fprintf (stderr, "\t      abbrev:  Short, abbreviated word (lower case only), like satfaa (%d char max)\n", MGD77_COL_ABBREV_LEN);
 		fprintf (stderr, "\t      name:    Descriptive name, like \"Geosat/ERS-1 Free-air gravity\" (%d char max)\n", MGD77_COL_NAME_LEN);
@@ -399,6 +418,11 @@ int main (int argc, char **argv)
 		if (file[0] == 'm' && file[1] == '\0') {
 			got_c = ADD_IGRF;
 		}
+#ifdef USE_CM4
+		else if (file[0] == '4' && file[1] == '\0') {
+			got_c = ADD_CM4;
+		}
+#endif
 		else if (file[0] == 'c' && file[1] == '\0') {
 			got_c = ADD_CARTER;
 			MGD77_carter_init (&Carter);	/* Initialize Carter machinery */
@@ -411,11 +435,25 @@ int main (int argc, char **argv)
 			got_c = ADD_RMAG;
 			MTF_col = mfield;
 		}
+#ifdef USE_CM4
+		else if (file[0] == 'R' && (file[1] == '\0' || ((mfield = (file[1] - '0')) >= 1 && mfield <= 2)) ) {
+			got_c = ADD_RMAG4;
+			MTF_col = mfield;
+		}
+#endif
 		else {
+#ifdef USE_CM4
+			fprintf(stderr, "%s: ERROR: -Ac expects 4, m, c, or g[1-4]\n", GMT_program);
+#else
 			fprintf(stderr, "%s: ERROR: -Ac expects m, c, or g[1-4]\n", GMT_program);
+#endif
 			exit (EXIT_FAILURE);
 		}
 	}
+	else if (got_e) {	/* Do E77 work by ignoring previous E77 settings */
+		In.use_flags[MGD77_M77_SET] = In.use_flags[MGD77_CDF_SET] = FALSE;		/* Turn use of flag bits OFF */
+		In.use_corrections[MGD77_M77_SET] = In.use_corrections[MGD77_CDF_SET] = FALSE;	/* Turn use of systematic corrections OFF */
+	}
 	else if (got_g) {	/* Read regular GMT grid */
 
 		GMT_err_fail (GMT_read_grd_info (file, &grd), file);
@@ -435,90 +473,18 @@ int main (int argc, char **argv)
 		interpolate = (threshold > 0.0);
 	}
 	else if (got_i) {	/* Read Sandwell/Smith IMG file */
-		struct STAT buf;
-		int min;
-		if (STAT (file, &buf)) {	/* Inquiry about file failed somehow */
-			fprintf (stderr, "%s: Unable to stat file %s\n", GMT_program, file);
-			exit (EXIT_FAILURE);
-		}
-		if (buf.st_size == (21600*12672*2))
-			min = 1;
-		else if (buf.st_size == (10800*6336*2))
-			min = 2;
-		else {
-			fprintf (stderr, "%s: img file %s does not seem to have 1 or 2 minute grid spacing.\n", GMT_program, file);
-			exit (EXIT_FAILURE);
-		}
-		parameters[IMG_SPACING] = (double)min;
-		if ((fp = GMT_fopen (file, "rb")) == NULL) {
-			fprintf (stderr, "%s: Error opening img file %s\n", GMT_program, file);
-			exit (EXIT_FAILURE);
-		}
-		GMT_grd_init (&grd, argc, argv, FALSE);
-		/* Select plain Mercator on a sphere with -Jm1 -R0/360/-72.006/+72.006 */
-		gmtdefs.ellipsoid = GMT_N_ELLIPSOIDS - 1;
-		project_info.units_pr_degree = TRUE;
-		project_info.pars[0] = 180.0;
-		project_info.pars[1] = 0.0;
-		project_info.pars[2] = 1.0;
-		project_info.projection = GMT_MERCATOR;
-		
-		GMT_err_fail (GMT_map_setup (0.0, 360.0, GMT_IMG_MINLAT, GMT_IMG_MAXLAT), "");
-		
-		GMT_geo_to_xy (project_info.w, project_info.s, &grd.x_min, &grd.y_min);
-		GMT_geo_to_xy (project_info.e, project_info.n, &grd.x_max, &grd.y_max);
-		grd.x_inc = grd.y_inc = parameters[IMG_SPACING] / 60.0;
-		grd.nx = irint (grd.x_max / grd.x_inc);
-		grd.ny = irint (grd.y_max / grd.y_inc);
-		if ((grd.nx * min) != 21600) {
-			fprintf (stderr, "%s: img file %s has %d x-nodes but expected %d\n", GMT_program, file, grd.nx, 21600/min);
-			exit (EXIT_FAILURE);
-		}
-		if ((grd.ny * min) != 12672) {
-			fprintf (stderr, "%s: img file %s has %d y-nodes but expected %d\n", GMT_program, file, grd.ny, 12672/min);
-			exit (EXIT_FAILURE);
-		}
-		grd.node_offset = 1;	/* These are pixel grids */
-		grd.xy_off = 0.5;
 		
+		GMT_read_img (file, &grd, &f, 0.0, 0.0, 0.0, 0.0, parameters[IMG_SCALE], (GMT_LONG)irint(parameters[IMG_MODE]), parameters[IMG_LAT], TRUE);
 		if (GMT_360_RANGE (grd.x_max, grd.x_min)) GMT_boundcond_parse (&edgeinfo, "g");
-	
 		GMT_boundcond_param_prep (&grd, &edgeinfo);
-	
-		/* Initialize bcr structure with 2 row/col boundaries:  */
-
-		GMT_bcr_init (&grd, GMT_pad, interpolant, threshold, &bcr);
-		
 		mx = grd.nx + GMT_pad[0] + GMT_pad[2];	my = grd.ny + GMT_pad[1] + GMT_pad[3];
-		track_decode = irint (parameters[IMG_MODE]);
 		interpolate = (threshold > 0.0);
-
-		f = (float *) GMT_memory (VNULL, (size_t)(mx * my), sizeof (float), GMT_program);
-		i2 = (short int *) GMT_memory (VNULL, (size_t)(grd.nx), sizeof (short int), GMT_program);
-		for (j = 0; j < grd.ny; j++) {	/* Read all the rows, offset by 2 boundary rows and cols */
-			ij = (j + GMT_pad[3]) * mx + GMT_pad[0];
-			fread ((void *)i2, sizeof (short int), (size_t)grd.nx, fp);
-			for (i = 0; i < grd.nx; i++) {	/* Process this row's values */
-				switch (track_decode) {
-					case 0:	/* No encoded track flags, do nothing */
-						break;
-					case 1:	/* Remove the track flag on odd (constrained) points */
-						if (i2[i]%2) i2[i]--;
-						break;
-					case 2:	/* Remove the track flag on odd (constrained) points and set unconstrained to NaN */
-						i2[i] = (i2[i]%2) ? i2[i] - 1 : SHRT_MIN;
-						break;
-					case 3:	/* Set odd (constrained) points to 1 and set unconstrained to 0 */
-						i2[i] %= 2;
-						break;
-				}
-				f[ij+i] = (float)((track_decode == 3) ? i2[i] : (i2[i] * parameters[IMG_SCALE]));
-			}
-		}
-		GMT_free ((void *)i2);
-		GMT_fclose (fp);
 	}
 	else if (got_table) {	/* Got a one- or two-column table to read */
+		int n_ave = 0;
+		double last_dnt = -DBL_MAX, sum_z = 0.0;
+		char *not_used = NULL;
+		
 		if (file[0] == '-') {   /* Just read from standard input */
 			fp = GMT_stdin;
 #ifdef SET_IO_MODE
@@ -533,13 +499,13 @@ int main (int argc, char **argv)
 		}
 
 		/* Skip any header records */
-		if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) GMT_fgets (line, BUFSIZ, fp);
+		if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) not_used = GMT_fgets (line, BUFSIZ, fp);
 
-		n = 0;
+		two_cols = (got_d || got_n || got_t);
+		n = (two_cols) ? -1 : 0;
 		n_alloc = GMT_CHUNK;
 		n_expected_fields = (GMT_io.ncol[GMT_IN]) ? GMT_io.ncol[GMT_IN] : BUFSIZ;
 		colvalue = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), GMT_program);
-		two_cols = (got_d || got_n || got_t);
 		if (two_cols) {	/* Got an abscissae column as well (dnt: d = dist, n = rec number, t = time) */
 			coldnt = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), GMT_program);
 			GMT_io.in_col_type[0] = kind;
@@ -582,12 +548,22 @@ int main (int argc, char **argv)
 				strcpy (tmp_string[n], word);
 			}
 			else if (two_cols) {
-				coldnt[n]   = in[0];
-				colvalue[n] = in[1];
+				if (in[0] > last_dnt) {	/* Start of new averaging scheme (if needed) */
+					if (n_ave) {
+						colvalue[n] = sum_z / n_ave;
+						coldnt[n]   = last_dnt;
+					}
+					n_ave = 0;
+					sum_z = 0.0;
+					n++;
+					last_dnt = in[0];
+				}
+				sum_z += in[1];	/* Add up possibly multiple values for same dnt */
+				n_ave++;
 			}
 			else
 				colvalue[n] = in[0];
-			n++;
+			if (!two_cols) n++;
 			if (n == (GMT_LONG)n_alloc) {
 				n_alloc <<= 1;
 				if (strings)
@@ -600,6 +576,7 @@ int main (int argc, char **argv)
 			n_fields = GMT_input (fp, &n_expected_fields, &in);
 		}
 		if (fp != GMT_stdin) GMT_fclose (fp);
+		if (two_cols && n_ave) { colvalue[n] = sum_z / n_ave; coldnt[n++] = last_dnt;}
 		if (!strings) colvalue = (double *) GMT_memory ((void *)colvalue, (size_t)n, sizeof (double), GMT_program);
 		if (two_cols) coldnt = (double *) GMT_memory ((void *)coldnt, (size_t)n, sizeof (double), GMT_program);
 	}
@@ -629,16 +606,14 @@ int main (int argc, char **argv)
 			exit (EXIT_FAILURE);
 		}
 
-		MGD77_Init_Ptr (MGD77_Header_Lookup, D->H.mgd77);
-		
 		/* Start reading data from file */
 	
 		column = MGD77_Get_Column (c_abbrev, &In);
 		set    = MGD77_Get_Set (c_abbrev);
 		
 		if (!got_e && column != MGD77_NOT_SET) {	/* A column with same abbreviation is already present in the file */
-			if (set == MGD77_M77_SET) {
-				fprintf (stderr, "%s: column %s is part of the standard MGD77 set and cannot be removed!\n", GMT_program, c_abbrev);
+			if (set == MGD77_M77_SET && !force) {
+				fprintf (stderr, "%s: column %s is part of the standard MGD77 set and cannot be removed unless you use -F!\n", GMT_program, c_abbrev);
 				exit (EXIT_FAILURE);
 			}
 			if (!replace) {
@@ -646,6 +621,8 @@ int main (int argc, char **argv)
 				MGD77_Free (D);	/* Free memory already allocated by MGD77_Read_File for this aborted effort */
 				continue;
 			}
+			n_dims = (D->H.info[In.order[column].set].col[In.order[column].item].constant) ? 0 : 1;
+			if (D->H.info[In.order[column].set].col[In.order[column].item].text) n_dims++;
 		}
 
 		if (delete) {	/* Must create a new file with everything except the fields to be deleted */
@@ -730,10 +707,10 @@ int main (int argc, char **argv)
 			}
 			n_changed++;
 		}
-		
+
 		if (got_c == ADD_IGRF) {	/* Append IGRF column */
 			int ix, iy, it;
-			double n_days, date, *xvar, *yvar, *tvar, IGRF[7];
+			double date, *xvar, *yvar, *tvar, IGRF[7];
 			
 			if ((ix = skip_if_missing ("lon",  list[argno], &In, D)) == MGD77_NOT_SET) continue;
 			if ((iy = skip_if_missing ("lat",  list[argno], &In, D)) == MGD77_NOT_SET) continue;
@@ -745,26 +722,74 @@ int main (int argc, char **argv)
 			colvalue = (double *)GMT_memory (VNULL, (size_t)D->H.n_records, sizeof (double), GMT_program);
 			
 			for (i = n_sampled = 0; i < D->H.n_records; i++) {
-				GMT_gcal_from_dt (tvar[i], &cal);			/* No adjust for TZ; this is GMT UTC time */
-				n_days = (GMT_is_gleap (cal.year)) ? 366.0 : 365.0;	/* Number of days in this year */
-				/* Get date as decimal year */
-				date = cal.year + cal.day_y / n_days + (cal.hour * GMT_HR2SEC_I + cal.min * GMT_MIN2SEC_I + cal.sec) * GMT_SEC2DAY;
+				date = MGD77_time_to_fyear (&In, tvar[i]);	/* Get date as decimal year */
 				colvalue[i] = (MGD77_igrf10syn (0, date, 1, 0.0, xvar[i], yvar[i], IGRF)) ? GMT_d_NaN : IGRF[MGD77_IGRF_F];
 				n_sampled++;
 			}
 			if (gmtdefs.verbose) fprintf (stderr, "%s: Estimated IGRF at %ld locations out of %ld for cruise %s\n", GMT_program, n_sampled, D->H.n_records, list[argno]);
 		}
+#ifdef USE_CM4	
+		else if (got_c == ADD_CM4) {	/* Append CM4 column */
+			int ix, iy, it;
+			double date, *xvar, *yvar, *tvar;
+			struct MGD77_CM4 CM4;
+			
+			if ((ix = skip_if_missing ("lon",  list[argno], &In, D)) == MGD77_NOT_SET) continue;
+			if ((iy = skip_if_missing ("lat",  list[argno], &In, D)) == MGD77_NOT_SET) continue;
+			if ((it = skip_if_missing ("time", list[argno], &In, D)) == MGD77_NOT_SET) continue;
+
+			xvar = (double *)D->values[ix];
+			yvar = (double *)D->values[iy];
+			tvar = (double *)D->values[it];
+			colvalue = (double *)GMT_memory (VNULL, (size_t)D->H.n_records, sizeof (double), GMT_program);
+			MGD77_CM4_init (&In, &CM4);
+			
+			for (i = n_sampled = 0; i < D->H.n_records; i++) {
+				date = MGD77_time_to_fyear (&In, tvar[i]);	/* Get date as decimal year */
+				colvalue[i] = MGD77_Calc_CM4 (date, xvar[i], yvar[i], FALSE, &CM4);
+				n_sampled++;
+			}
+			MGD77_CM4_end (&CM4);
+			if (gmtdefs.verbose) fprintf (stderr, "%s: Estimated CM4 at %ld locations out of %ld for cruise %s\n", GMT_program, n_sampled, D->H.n_records, list[argno]);
+		}
+		else if (got_c == ADD_RMAG4) {	/* Append recomputed residual mag column */
+			int ix, iy, it, im;
+			double date, *xvar, *yvar, *tvar, *mvar, IGRF[7];
+			char field[5];
+			
+			if ((ix = skip_if_missing ("lon",  list[argno], &In, D)) == MGD77_NOT_SET) continue;
+			if ((iy = skip_if_missing ("lat",  list[argno], &In, D)) == MGD77_NOT_SET) continue;
+			if ((it = skip_if_missing ("time", list[argno], &In, D)) == MGD77_NOT_SET) continue;
+			sprintf (field, "mtf%d", MTF_col);
+			if ((im = skip_if_missing (field, list[argno], &In, D)) == MGD77_NOT_SET) continue;
+
+			xvar = (double *)D->values[ix];
+			yvar = (double *)D->values[iy];
+			tvar = (double *)D->values[it];
+			mvar = (double *)D->values[im];
+			colvalue = (double *)GMT_memory (VNULL, (size_t)D->H.n_records, sizeof (double), GMT_program);
+			
+			for (i = n_sampled = 0; i < D->H.n_records; i++) {
+				date = MGD77_time_to_fyear (&In, tvar[i]);	/* Get date as decimal year */
+/* Change this--> */		check = MGD77_igrf10syn (0, date, 1, 0.0, xvar[i], yvar[i], IGRF);
+				colvalue[i] = (check) ? GMT_d_NaN : mvar[i] - IGRF[MGD77_IGRF_F];
+				n_sampled++;
+			}
+			if (gmtdefs.verbose) fprintf (stderr, "%s: Estimated recomputed magnetic anomaly at %ld locations out of %ld for cruise %s\n", GMT_program, n_sampled, D->H.n_records, list[argno]);
+		}
+#endif
 		else if (got_c == ADD_GRAV) {	/* Append IGF column */
-			int ix, iy;
+			int ix, iy, use;
 			double *xvar, *yvar;
 			
 			if ((ix = skip_if_missing ("lon", list[argno], &In, D)) == MGD77_NOT_SET) continue;
 			if ((iy = skip_if_missing ("lat", list[argno], &In, D)) == MGD77_NOT_SET) continue;
 
 			if (GF_version == MGD77_NOT_SET) {
-				GF_version = D->H.mgd77->Gravity_Theoretical_Formula_Code - '0';
+				use = (In.original) ? MGD77_ORIG : MGD77_REVISED;
+				GF_version = D->H.mgd77[use]->Gravity_Theoretical_Formula_Code - '0';
 				if (GF_version < MGD77_IGF_HEISKANEN || GF_version > MGD77_IGF_1980) {
-					fprintf (stderr, "%s: Invalid Gravity Theoretical Formula Code (%c) - default to %d\n", GMT_program, D->H.mgd77->Gravity_Theoretical_Formula_Code, MGD77_IGF_1980);
+					fprintf (stderr, "%s: Invalid Gravity Theoretical Formula Code (%c) - default to %d\n", GMT_program, D->H.mgd77[use]->Gravity_Theoretical_Formula_Code, MGD77_IGF_1980);
 					GF_version = MGD77_IGF_1980;
 				}
 			}
@@ -793,7 +818,7 @@ int main (int argc, char **argv)
 		}
 		else if (got_c == ADD_RMAG) {	/* Append recomputed residual mag column */
 			int ix, iy, it, im;
-			double n_days, date, *xvar, *yvar, *tvar, *mvar, IGRF[7];
+			double date, *xvar, *yvar, *tvar, *mvar, IGRF[7];
 			char field[5];
 			
 			if ((ix = skip_if_missing ("lon",  list[argno], &In, D)) == MGD77_NOT_SET) continue;
@@ -809,10 +834,7 @@ int main (int argc, char **argv)
 			colvalue = (double *)GMT_memory (VNULL, (size_t)D->H.n_records, sizeof (double), GMT_program);
 			
 			for (i = n_sampled = 0; i < D->H.n_records; i++) {
-				GMT_gcal_from_dt (tvar[i], &cal);			/* No adjust for TZ; this is GMT UTC time */
-				n_days = (GMT_is_gleap (cal.year)) ? 366.0 : 365.0;	/* Number of days in this year */
-				/* Get date as decimal year */
-				date = cal.year + cal.day_y / n_days + (cal.hour * GMT_HR2SEC_I + cal.min * GMT_MIN2SEC_I + cal.sec) * GMT_SEC2DAY;
+				date = MGD77_time_to_fyear (&In, tvar[i]);	/* Get date as decimal year */
 				check = MGD77_igrf10syn (0, date, 1, 0.0, xvar[i], yvar[i], IGRF);
 				colvalue[i] = (check) ? GMT_d_NaN : mvar[i] - IGRF[MGD77_IGRF_F];
 				n_sampled++;
@@ -968,7 +990,7 @@ int main (int argc, char **argv)
 			 * all the verification steps has been taken to make this E77 a correction file
 			 */
 			 
-			P = D->H.mgd77;
+			P = D->H.mgd77[MGD77_ORIG];	/* Because E77 is absolute and not incremental we start from original settings */
 			if (!GMT_fgets (line, BUFSIZ, fp_e)) {
 				fprintf(stderr, "%s: ERROR: Could not read record #1 from %s.e77 - aborting\n", GMT_program, list[argno]);
 				e_error++;
@@ -1000,7 +1022,7 @@ int main (int argc, char **argv)
 				GMT_chop (line);		/* Rid the world of CR/LF */
 				if (!strncmp (line, "Y Errata table verification status", (size_t)34)) verified = TRUE;
 			}
-			if (!verified) {
+			if (!verified && !ignore_verify) {
 				fprintf(stderr, "%s: ERROR: E77 file %s not yet verified.  E77 not applied\n", GMT_program, efile);
 				e_error++;
 			}
@@ -1019,7 +1041,7 @@ int main (int argc, char **argv)
 			while (GMT_fgets (line, BUFSIZ, fp_e)) {
 				if (line[0] == '#' || line[0] == '\n') continue;	/* Comments or blank lines are OK */
 				if (line[1] == '-') {		/* Header record */
-					if (!(line[0] == 'Y' || line[0] == 'N')) {		/* Unprocessed recommendation? */
+					if (!(line[0] == 'Y' || line[0] == 'N') && !ignore_verify) {		/* Unprocessed recommendation? */
 						fprintf (stderr, "%s: UNDECIDED: %s\n", list[argno], line);
 						if (line[0] == '?') n_unprocessed++;
 						e_error++;
@@ -1036,11 +1058,13 @@ int main (int argc, char **argv)
 			
 			if (e_error) {
 				fprintf(stderr, "%s: ERROR: The file %s has too many errors.  E77 not applied\n", GMT_program, efile);
+				GMT_fclose (fp_e);
 				MGD77_Free (D);	/* Free memory allocated by MGD77_Read_File for this aborted effort */
 				continue;
 			}
 			if (n_unprocessed) {
 				fprintf(stderr, "%s: ERROR: The file %s has unprocessed E77 recommendations.  E77 not applied\n", GMT_program, efile);
+				GMT_fclose (fp_e);
 				MGD77_Free (D);	/* Free memory allocated by MGD77_Read_File for this aborted effort */
 				continue;
 			}
@@ -1069,24 +1093,24 @@ int main (int argc, char **argv)
 					continue;
 				}
 				if (field[0] == 'H') {
-					type = 0;
+					type = E77_HEADER_MODE;
 					number = (GMT_LONG)atoi (&field[1]);
 				}
 				else {
 					type = 1;
 					number = (GMT_LONG)item;
 				}
+				if (e77_skip_mode[type]) continue;
+				if (!e77_skip_mode[type] && YorN == 'N') continue;
 				if (kind == 'W') {	/* Output the warning (if Y) and goto next line*/
-					if (YorN == 'Y') fprintf (stderr, "%s: WARNING: %s\n", list[argno], line);
+					if (gmtdefs.verbose == 2 && (YorN == 'Y' || (ignore_verify && YorN == '?'))) fprintf (stderr, "%s: WARNING: %s\n", list[argno], line);
 					continue;
 				}
-				if (e77_mode[type] == -1) continue;
-				if (e77_mode[type] == 0 && YorN == 'N') continue;
 				if (!got_default_answer (line, answer)) continue;
 				
 				/* Here we must do something */
 				
-				if (type == 0) {	/* Header meta data fixes */
+				if (type == E77_HEADER_MODE) {	/* Header meta data fixes */
 				
 					key = MGD77_Param_Key (number, item);	/* Returns -ve if sequence not found or item not found, >=0 otherwise */
 					
@@ -1098,19 +1122,9 @@ int main (int argc, char **argv)
 							fprintf (stderr, "%s: Warning: Sequence number %ld, Item %d is not supported - skipped\n", GMT_program, number, item);
 							break;
 						default:	/* Update internal structure as well as netCDF file attributes */
-							if (key == (MGD77_N_HEADER_PARAMS-1)) {	/* Could be any of records 18-24 */
-								char att[32];
-								length = strlen (answer);
-								item = number - 17;
-								strncpy (P->Additional_Documentation[item], answer, (size_t)length);
-								sprintf (att, "%s_%d", MGD77_Header_Lookup[key].name, item);
-								MGD77_Put_Param (&In, att, length, P->Additional_Documentation[item]);
-							}
-							else {
-								length = (MGD77_Header_Lookup[key].length == 1) ? 1 : strlen (answer);
-								strncpy (MGD77_Header_Lookup[key].ptr, answer, (size_t)length);
-								MGD77_Put_Param (&In, MGD77_Header_Lookup[key].name, length, MGD77_Header_Lookup[key].ptr);
-							}
+							length = (MGD77_Header_Lookup[key].length == 1) ? 1 : strlen (answer);
+							strncpy (MGD77_Header_Lookup[key].ptr[MGD77_REVISED], answer, (size_t)length);
+							MGD77_Put_Param (&In, MGD77_Header_Lookup[key].name, length, MGD77_Header_Lookup[key].ptr[MGD77_ORIG], length, MGD77_Header_Lookup[key].ptr[MGD77_REVISED], 2);
 							n_E77_headers++;
 							break;
 					}
@@ -1187,8 +1201,10 @@ int main (int argc, char **argv)
 			has_time = TRUE;
 			if ((it = skip_if_missing ("time", list[argno], &In, D)) == MGD77_NOT_SET)
 				has_time = FALSE;
-			else
+			else {	/* See if we really have time or if they are all NaN */
 				tvar = (double *)D->values[it];
+				for (rec = 0, has_time = FALSE; !has_time && rec < D->H.n_records; rec++) if (!GMT_is_dnan (tvar[rec])) has_time = TRUE;
+			}
 			while (GMT_fgets (line, BUFSIZ, fp_e)) {	/* Read until EOF */
 				sscanf (line, "%c %s %s %ld %s", &YorN, ID, timestamp, &rec, code);
 				if (strcmp (In.NGDC_id, ID)) {
@@ -1196,32 +1212,34 @@ int main (int argc, char **argv)
 					e_error++;
 					continue;
 				}
-				if (YorN == 'N') continue;	/* Already decided NOT to use this correction */
-				if (YorN == '?') {		/* Undecided: Output the warning and goto next line*/
+				if (YorN == 'N') continue;			/* Already decided NOT to use this correction */
+				if (YorN == '?' && !ignore_verify) {		/* Undecided: Output the warning and goto next line unless we ignore verification */
 					fprintf (stderr, "%s: UNDECIDED: %s\n", list[argno], line);
 					continue;
 				}
-				/* Here, YorN is 'Y' */
+				/* Here, YorN is 'Y' (or '?' if ignore_verify is TRUE) */
 				rec--;	/* E77 starts with rec = 1 for first data record */
 				if (has_time) {
-					if (GMT_verify_expectations (GMT_IS_ABSTIME, GMT_scanf (timestamp, GMT_IS_ABSTIME, &rec_time), timestamp)) {
-						if (!strcmp(timestamp,"NaN")) fprintf (stderr, "%s: WARNING: E77 time stamp %s, using recno\n", GMT_program, timestamp);
-						else {
-							fprintf (stderr, "%s: ERROR: E77 time stamp (%s) in wrong format? - skipped\n", GMT_program, timestamp);
+					if (!strcmp(timestamp,"NaN")) {
+						if (gmtdefs.verbose == 2) fprintf (stderr, "%s: WARNING: %s: E77 time stamp %s, using recno\n", GMT_program, ID, timestamp);
+					}
+					else {	/* Must try to interpret the timestamp */
+						if (GMT_verify_expectations (GMT_IS_ABSTIME, GMT_scanf (timestamp, GMT_IS_ABSTIME, &rec_time), timestamp)) {
+							fprintf (stderr, "%s: ERROR: %s: E77 time stamp (%s) in wrong format? - skipped\n", GMT_program, ID, timestamp);
+							continue;
+						}
+						del_t = fabs (tvar[rec] - rec_time);
+						if (del_t > (0.06 + GMT_CONV_LIMIT)) {	/* 0.06 is finest time step in MGD77 file so we allow that much slop */
+							fprintf (stderr, "%s: ERROR: %s: E77 time stamp and record number do not match record time (del_t = %g s) - skipped\n", GMT_program, ID, del_t);
 							continue;
 						}
-					}
-					del_t = fabs (tvar[rec] - rec_time);
-					if (del_t > (0.06 + GMT_CONV_LIMIT)) {	/* 0.06 is finest time step in MGD77 file so we allow that much slop */
-						fprintf (stderr, "%s: ERROR: E77 time stamp and record number do not match record time (del_t = %g s) - skipped\n", GMT_program, del_t);
-						continue;
 					}
 				}
 				pos = 0;
 				item = -1;	/* So we increment to 0 inside the loop */
 				while (GMT_strtok (code, "-", &pos, p)) {
 					item++;
-					if (e77_mode[item+2]) continue;	/* Ignore this sort of code */
+					if (e77_skip_mode[item+2]) continue;	/* Ignore this sort of code */
 					if (p[0] == '0') continue;
 					for (k = 0; k < (int)strlen(p); k++) {	/* Loop over one or more codes */
 					
@@ -1232,7 +1250,7 @@ int main (int argc, char **argv)
 									n_E77_flags++;
 									break;
 								case 'B':
-									fprintf (stderr, "%s: Decreasing time %s - Source Institution need to sort records\n", list[argno], timestamp);
+									if (gmtdefs.verbose == 2) fprintf (stderr, "%s: Decreasing time %s - Source Institution need to sort records\n", list[argno], timestamp);
 									break;
 								case 'C':	/* Excessive speed - flag time, lon, lat */
 									flags[rec] |= set_bit(NCPOS_TIME);
@@ -1292,8 +1310,12 @@ int main (int argc, char **argv)
 			old_flags =  (nc_inq_varid (In.nc_id, "MGD77_flags", &var_id) == NC_NOERR);	/* TRUE if flag variable exists already */
 			
 			if (n_E77_flags) {	/* Add flags to netCDF file */
-				if (old_flags)	/* Flag variable exists already - simply replace existing flags with the new ones */
-					memcpy ((void *)D->flags[0], (void *)flags, (size_t)(D->H.n_records * sizeof (int)));
+				if (old_flags) {	/* Flag variable exists already - simply replace existing flags with the new ones */
+					if (D->flags[0])	/* Was allocated and read */
+						memcpy ((void *)D->flags[0], (void *)flags, (size_t)(D->H.n_records * sizeof (int)));
+					else	/* Was not allcoated */
+						D->flags[0] = flags;
+				}
 				else {	/* We need to define the flags for the first time */
 					dims[0] = In.nc_recid;
 					MGD77_nc_status (nc_def_var (In.nc_id, "MGD77_flags", NC_INT, 1, dims, &var_id));	/* Define an array variable */
@@ -1467,14 +1489,11 @@ int main (int argc, char **argv)
 int decode_A_options (int mode, char *line, char *file, double parameters[])
 {
 	int error = 0, n;
-	char txt_a[GMT_TEXT_LEN], txt_b[GMT_TEXT_LEN];
 	
-	if (mode == 1) {	/* For *.img files since we need to know data scale and grid mode : fname/scl/mode */
-		/* -A[+]i<spacing>/<scale>/<mode>/<filename> */
-		n = sscanf (line, "%[^/]/%[^/]/%s", txt_a, txt_b, file);
-		if (n != 3) error = 1;
-		parameters[IMG_SCALE]   = atof (txt_a);
-		parameters[IMG_MODE]    = atof (txt_b);
+	if (mode == 1) {	/* For *.img files since we need to know data scale and grid mode */
+		/* -A[+]i<filename>,<scale>/<mode>[/<lat>] */
+		n = sscanf (line, "%[^,],%lf,%lf,%lf", file, &parameters[IMG_SCALE], &parameters[IMG_MODE], &parameters[IMG_LAT]);
+		if (n < 3) error = 1;
 	}
 	else {	/* GMT grid or table: No data scale and mode to worry about */
 		/* -A[+]a|c|d|D|e|g|n|t|T<filename> */
@@ -1486,7 +1505,8 @@ int decode_A_options (int mode, char *line, char *file, double parameters[])
 
 int decode_I_options (char *line, char *abbrev, char *name, char *units, char *size, char *comment, double parameters[])
 {	/* -I<abbrev>/<name>/<units>/<size>/<scale>/<offset>/\"comment\" */
-	int i = 0, k, pos = 0, error;
+	int i = 0, k, error;
+	GMT_LONG pos = 0;
 	char p[BUFSIZ];
 	
 	while (i < 7 && GMT_strtok (line, "/", &pos, p)) {	/* Process the 7 items */
