@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------
- *	$Id: makepattern.c,v 1.24 2009/01/09 04:02:35 guru Exp $
+ *	$Id: makepattern.c,v 1.25 2009/07/11 03:16:31 guru Exp $
  *
  *      Copyright (c) 1999-2009 by P. Wessel
  *      See COPYING file for copying and redistribution conditions.
@@ -273,7 +273,7 @@ unsigned short int *get_icon (char *file, int *nx, int *ny, int invert)
 	int i, n_items, n_int, j, k, n_read, n_alloc, n_lines, last;
 	unsigned int integer;
 	unsigned short int *pattern;
-	char t[8][7], line[80], width[12], height[12];
+	char t[8][7], line[80], width[12], height[12], *not_used = NULL;
 	FILE *fp;
 
 	if ((fp = fopen (file, "r")) == NULL) {
@@ -281,14 +281,14 @@ unsigned short int *get_icon (char *file, int *nx, int *ny, int invert)
 		return ((unsigned short int *)NULL);
 	}
 
-	fgets (line, 80, fp);
+	not_used = fgets (line, 80, fp);
 	line[strlen(line)-1] = 0;
 	sscanf (&line[3], "%*[^,], %[^,], %[^,]", width, height);
 	i = 0;	while (width[i] != '=') i++;
 	*nx = atoi (&width[i+1]);
 	i = 0;	while (height[i] != '=') i++;
 	*ny = atoi (&height[i+1]);
-	fgets (line, 80, fp);
+	not_used = fgets (line, 80, fp);
 	line[strlen(line)-1] = 0;
 	n_items = (*nx) * (*ny) / 4;
 	n_int = n_items / 4;
@@ -296,14 +296,14 @@ unsigned short int *get_icon (char *file, int *nx, int *ny, int invert)
 	pattern = (unsigned short int *) malloc ((size_t)n_alloc);
 	n_lines = n_items / 32;
 	for (i = k = 0; i < n_lines; i++) {
-		fgets (line, 80, fp);
+		not_used = fgets (line, 80, fp);
 		line[strlen(line)-1] = 0;
 		last = (i == (n_lines - 1));
 		if (last) {
 			n_read = sscanf (&line[1], "%[^,],%[^,],%[^,],%[^,],%[^,],%[^,],%[^,],%[^,]", t[0], t[1], t[2], t[3], t[4], t[5], t[6], t[7]);
 			if (n_read == 4) {
 				sscanf (&line[1], "%[^,], %[^,], %[^,], %[^,]", t[0], t[1], t[2], t[3]);	/* New style icon file */
-				fgets (line, 80, fp);
+				not_used = fgets (line, 80, fp);
 				line[strlen(line)-1] = 0;
 				sscanf (&line[1], " %[^,], %[^,], %[^,], %[^,]", t[4], t[5], t[6], t[7]);	/* New style icon file */
 			}
@@ -312,7 +312,7 @@ unsigned short int *get_icon (char *file, int *nx, int *ny, int invert)
 			n_read = sscanf (&line[1], "%[^,],%[^,],%[^,],%[^,],%[^,],%[^,],%[^,],%[^,],", t[0], t[1], t[2], t[3], t[4], t[5], t[6], t[7]);
 			if (n_read == 4) {
 				sscanf (&line[1], "%[^,], %[^,], %[^,], %[^,],", t[0], t[1], t[2], t[3]);	/* New style icon file */
-				fgets (line, 80, fp);
+				not_used = fgets (line, 80, fp);
 				line[strlen(line)-1] = 0;
 				sscanf (&line[1], " %[^,], %[^,], %[^,], %[^,],", t[4], t[5], t[6], t[7]);	/* New style icon file */
 			}
