@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: pssegy.c,v 1.26 2008/03/24 08:58:33 guru Exp $
+ *	$Id: pssegy.c,v 1.29 2009/07/10 00:23:54 guru Exp $
  *
- *    Copyright (c) 1999-2008 by T. Henstock
+ *    Copyright (c) 1999-2009 by T. Henstock
  *    See README file for copying and redistribution conditions.
  *--------------------------------------------------------------------*/
 /* pssegy program to plot segy files in postscript with variable trace spacing option
@@ -22,6 +22,9 @@
  *
  *               2.1 10/4/2001 fix unreduce bug, modify to byte-swap if necessary, make 64-bit safe
  *
+ *                   10/7/2009 fix bug when reading trace locations from arbitrary header locations,
+ *                             8 bytes copied, should be 4 bytes 
+ * 
  * This program is free software and may be copied or redistributed under the terms
  * of the GNU public license.
  */
@@ -68,7 +71,8 @@ int main (int argc, char **argv)
 	float *data;
 	SEGYHEAD *header;
 	char *head;
-	long *head2;
+	/* long *head2;  edited to correct bug when reading trace locations from arbitrary trace header locations*/
+	int head2;
 	SEGYREEL binhead;
 
 	FILE *fpi = NULL, *fpt = NULL;
@@ -164,7 +168,7 @@ int main (int argc, char **argv)
 				case 'T':
 					strcpy (input, &argv[i][2]);
 					if ((trace_file = strlen(input))){
-						if ((fpt = GMT_fopen (input, "r")) == NULL){
+						if ((fpt = fopen (input, "r")) == NULL){
 							fprintf(stderr, "%s: Cannot find trace list file %s\n", GMT_program, input);
 							error++;
 						}
@@ -185,7 +189,7 @@ int main (int argc, char **argv)
 					break;
 			}
 		}
-		else if ((fpi = GMT_fopen (argv[i], "rb")) == NULL) {
+		else if ((fpi = fopen (argv[i], "rb")) == NULL) {
 			fprintf (stderr, "%s: Cannot find segy file %s\n", GMT_program,argv[i]);
 			exit (EXIT_FAILURE);
 		}
@@ -368,11 +372,10 @@ use a few of these*/
 			x0 = (double) cdpval;
 		}
 		else if (byte_x){ /* ugly code - want to get value starting at byte_x of header into a double... */
-		   	long plot_loc;
 			head = (char *) header;
-			head2 = (long *) &(head[byte_x]);
-			plot_loc = ((swap_bytes)? GMT_swab4(*head2): *head2);
-		  	x0 = (double) plot_loc;
+			memcpy(&head2, &head[byte_x], 4); /* edited to fix bug where 8bytes were copied from head.
+												Caused by casting to a long directly from char array*/ 
+			x0 = (double) ((swap_bytes)? GMT_swab4(head2): head2);
 		}
 		else
 			x0 = (1.0 + (double) ix);
