@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------
- *	$Id: pssegyz.c,v 1.34 2009/01/09 04:02:35 guru Exp $
+ *	$Id: pssegyz.c,v 1.35 2009/07/10 00:23:54 guru Exp $
  *
  *    Copyright (c) 1999-2009 by T. Henstock
  *    See README file for copying and redistribution conditions.
@@ -78,7 +78,8 @@ int main (int argc, char **argv)
 	char input[512];
 	char *text;
 	char *head;
-	long *head2;
+	/* long *head2;  edited to correct bug when reading trace locations from arbitrary trace header locations*/
+	int head2;
 
 
 	char reelhead[3200];
@@ -410,11 +411,10 @@ use a few of these*/
 			x0 = (double) cdpval;
 		}
 		else if (byte_x){ /* ugly code - want to get value starting at byte_x of header into a double... */
-			long plot_loc;
 		   	head = (char *) header;
-			head2 = (long *) &(head[byte_x]);
-			plot_loc = ((swap_bytes)? GMT_swab4(*head2): *head2);
-		  	x0 = (double) plot_loc;
+			memcpy(&head2, &head[byte_x], 4); /* edited to fix bug where 8bytes were copied from head.
+                                                Caused by casting to a long directly from char array*/ 
+			x0 = (double) ((swap_bytes)? GMT_swab4(head2): head2);
 		}
 		else if (fix_x)
 			x0 /= dx;
@@ -431,11 +431,10 @@ use a few of these*/
 			y0 = (double) cdpval;
 		}
 		else if (byte_y){
-		  	long plot_loc;
 			head =  (char *) header;
-			head2 = (long *) &(head[byte_y]);
-			plot_loc = ((swap_bytes)? GMT_swab4(*head2): *head2);
-			y0 = (double) plot_loc;
+			memcpy(&head2, &head[byte_y], 4); /* edited to fix bug where 8bytes were copied from head.
+                                                Caused by casting to a long directly from char array*/ 
+			y0 = (double) ((swap_bytes)? GMT_swab4(head2): head2);
 		}
 		else if (fix_y)
 			y0 /= dx;
