@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: segy2grd.c,v 1.27 2008/03/22 11:55:37 guru Exp $
+ *	$Id: segy2grd.c,v 1.34 2009/07/10 00:23:54 guru Exp $
  *
- *	Copyright (c) 1991-2008 by T. Henstock
+ *	Copyright (c) 1991-2009 by T. Henstock
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -30,7 +30,7 @@
 
 int main (int argc, char **argv)
 {
-	GMT_LONG error = FALSE, got_input = FALSE, pixel = FALSE, read_cont = FALSE, count = FALSE, average = TRUE;
+	GMT_LONG error = FALSE, got_input = FALSE, read_cont = FALSE, count = FALSE, average = TRUE;
 	GMT_LONG nodata_set = FALSE, read_cdp = FALSE, read_offset = FALSE;
 	GMT_LONG swap_bytes = !WORDS_BIGENDIAN;
 
@@ -38,6 +38,7 @@ int main (int argc, char **argv)
 	GMT_LONG n_empty = 0, n_stuffed = 0, n_bad = 0, n_confused = 0;
 	GMT_LONG check, n_traces=10000, n_samp=0, n_sampr=0, byte_x = 0, ix, isamp, ij0;
 
+	int pixel = FALSE;
 	double w, e, s, n, dx = 0.0, dy = 0.0, xy_off, idy, no_data_d;
 	double s_int = 0.0, x_coord_scale=1.0, x0, yval;
 
@@ -54,7 +55,8 @@ int main (int argc, char **argv)
 	float *data;
 	SEGYHEAD *header;
 	char *head;
-	long *head2;
+	/* long *head2;  edited to correct bug when reading trace locations from arbitrary trace header locations*/
+	int head2;
 	SEGYREEL binhead;
 
 
@@ -147,7 +149,7 @@ int main (int argc, char **argv)
 					break;
 			}
 		}
-		else if ((fpi = GMT_fopen (argv[i], "rb")) == NULL) {
+		else if ((fpi = fopen (argv[i], "rb")) == NULL) {
 			fprintf (stderr, "%s: Cannot find segy file %s\n", GMT_program,argv[i]);
 			exit (EXIT_FAILURE);
 		}
@@ -184,6 +186,7 @@ int main (int argc, char **argv)
 		exit (EXIT_FAILURE);
 	}
 
+	GMT_check_lattice (&dx, &dy, &pixel, NULL);
 
 	read_cont = (!read_cdp && !read_offset && !byte_x);
 	if (!project_info.region_supplied) {
@@ -227,7 +230,7 @@ int main (int argc, char **argv)
 
 	if (gmtdefs.verbose) fprintf (stderr, "%s: nx = %d  ny = %d\n", GMT_program, grd.nx, grd.ny);
 
-	nm = grd.nx * grd.ny;
+	nm = GMT_get_nm (grd.nx, grd.ny);
 
 	a = (float *) GMT_memory (VNULL, (size_t)nm, sizeof (float), GMT_program);
 	flag = (GMT_LONG *) GMT_memory (VNULL, (size_t)nm, sizeof (GMT_LONG), GMT_program);
@@ -348,11 +351,10 @@ use a few of these*/
 				x0 = (double) cdpval;
 			}
 			else if (byte_x){ /* ugly code - want to get value starting at byte_x of header into a double... */
-		 	  	long plot_loc;
 				head = (char *) header;
-				head2 = (long *) &(head[byte_x]);
-				plot_loc = ((swap_bytes)? GMT_swab4(*head2): *head2);
-		 	 	x0 = (double) plot_loc;
+			memcpy(&head2, &head[byte_x], 4); /* edited to fix bug where 8bytes were copied from head.
+                                                Caused by casting to a long directly from char array*/ 
+			x0 = (double) ((swap_bytes)? GMT_swab4(head2): head2);
 			}
 			else
 				x0 = (1.0 + (double) ix);
@@ -427,7 +429,7 @@ overlaps a defined header in a strange way */
 			(GMT_is_dnan (no_data_d)) ? fprintf (stderr, "NaN\n") : fprintf (stderr, line, no_data_d);
 			if (n_bad) fprintf (stderr, "%s: %ld records unreadable\n", GMT_program, n_bad);
 			if (n_stuffed) fprintf (stderr, "%s: Warning - %ld nodes had multiple entries that were averaged\n", GMT_program, n_stuffed);
-			if (n_confused) fprintf (stderr, "%s: Warning - %ld values gave bad indeces: Pixel vs gridline confusion?\n", GMT_program, n_confused);
+			if (n_confused) fprintf (stderr, "%s: Warning - %ld values gave bad indices: Pixel vs gridline confusion?\n", GMT_program, n_confused);
 		}
 	}
 
