@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: backtracker.c,v 1.32 2007/08/31 20:26:10 guru Exp $
+ *	$Id: backtracker.c,v 1.46 2009/07/11 03:16:31 guru Exp $
  *
- *   Copyright (c) 1999-2007 by P. Wessel
+ *   Copyright (c) 1999-2009 by P. Wessel
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -66,20 +66,20 @@ int main (int argc, char **argv)
 {
 	struct EULER *p;		/* Pointer to array of stage poles */
 
-	int n_points;			/* Number of data points read */
-	int n_chunk;			/* Total length or array returned by libeuler functions */
-	int n_track;			/* Number of points in a track segment */
-	int n_stages;			/* Number of stage poles */
-	int n_segments;			/* Number of path segments written out */
-	int n_skipped = 0;		/* Number of points skipped because t < 0 */
+	GMT_LONG n_points;			/* Number of data points read */
+	GMT_LONG n_chunk;			/* Total length or array returned by libeuler functions */
+	GMT_LONG n_track;			/* Number of points in a track segment */
+	GMT_LONG n_stages = 0;			/* Number of stage poles */
+	GMT_LONG n_segments;			/* Number of path segments written out */
+	GMT_LONG n_skipped = 0;		/* Number of points skipped because t < 0 */
 	int track_limit = 0;		/* Flag for limiting the output track extent [no limit] */
 	int n_args;
 	int n_files = 0, fno;
 	int n_fields, n_expected_fields;
-	int n_read = 0;
+	GMT_LONG n_read = 0;
 	int n_out;
 
-	int i, j, k;			/* Misc. counters */
+	GMT_LONG i, j, k;			/* Misc. counters */
 
 	BOOLEAN error = FALSE;		/* Set to TRUE if arguments are inconsistent */
 	BOOLEAN flowline = FALSE;	/* TRUE means we want flowlines, FALSE we want hotspot tracks */
@@ -94,6 +94,8 @@ int main (int argc, char **argv)
 	BOOLEAN nofile = TRUE;
 	BOOLEAN confidence = FALSE;
 	BOOLEAN finite_out = FALSE;	/* FALSE unless -W is set */
+	BOOLEAN single_rotation = FALSE;	/* FALSE unless -F is set */
+	BOOLEAN do_track = FALSE;	/* TRUE if -L is used */
 
 	double d_km = 0.0;		/* Step interval along calculated tracks */
 	double t_zero = 0.0;		/* Current age in Ma */
@@ -104,6 +106,9 @@ int main (int argc, char **argv)
 	double age;			/* Age of seamount, in Ma */
 	double *in, *out;		/* i/o arrays used by GMT */
 	double t_low, t_high;		/* upper/lower age/stage to ouput for track segments */
+	double rot_lon, rot_lat, rot_w;	/* A finite rotation */
+	double R[3][3];			/* Rotation matrix */
+	double x[3], y[3];		/* Two 3-D unit vectors */
 
 	char *euler_file = CNULL;	/* Name pointer for file with stage poles */
 	char *file_stem = CNULL;	/* Name pointer for file stem for individual track files */
@@ -111,11 +116,13 @@ int main (int argc, char **argv)
 	char type[50];			/* What kind of line (flowline or hotspot track) */
 	char dir[8];			/* From or To */
 	char conf_flag = 0;		/* t for time, a for angle, blank for nothing */
+	char txt_a[GMT_TEXT_LEN], txt_b[GMT_TEXT_LEN];
+	char *not_used = NULL;
 
 	FILE *fp = NULL;		/* File pointer for input data */
 	FILE *fpo = NULL;		/* File pointer for output data */
 
-	PFI spot_func;			/* Pointer to the required forth/back track function */
+	PFI spot_func = NULL;			/* Pointer to the required forth/back track function */
 
 	out = (double *)NULL;
 
@@ -135,8 +142,10 @@ int main (int argc, char **argv)
 				/* Common parameters */
 
 				case 'H':
+				case 'M':
 				case 'V':
 				case ':':
+				case 'm':
 				case '\0':
 					error += GMT_parse_common_options (argv[i], NULL, NULL, NULL, NULL);
 					break;
@@ -179,6 +188,7 @@ int main (int argc, char **argv)
 					break;
 
 				case 'L':	/* Specify what kind of track to project */
+					do_track = TRUE;
 					switch (argv[i][2]) {
 						case 'F':	/* Calculate flowlines */
 							stage_id = TRUE;
@@ -202,8 +212,11 @@ int main (int argc, char **argv)
 					euler_file  = &argv[i][2];
 					break;
 
-				case 'M':               /* Multiple line segments */
-					GMT_multisegment (&argv[i][2]);
+				case 'e':	/* Apply a fixed total reconstruction rotation to all input points  */
+					single_rotation  = TRUE;
+					sscanf (&argv[i][2], "%[^/]/%[^/]/%lg", txt_a, txt_b, &rot_w);
+					error += GMT_verify_expectations (GMT_io.in_col_type[0], GMT_scanf_arg (txt_a, GMT_io.in_col_type[0], &rot_lon), txt_a);
+					error += GMT_verify_expectations (GMT_io.in_col_type[1], GMT_scanf_arg (txt_b, GMT_io.in_col_type[1], &rot_lat), txt_b);
 					break;
 
 				case 'Q':	/* Fixed age for all points */
@@ -247,26 +260,26 @@ int main (int argc, char **argv)
 
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
 		fprintf (stderr, "%s %s - Forward and backward flowlines and hotspot tracks\n\n", GMT_program, SPOTTER_VERSION);
-		fprintf (stderr, "usage: %s [infile(s)] -E<euler.d> [-A[young/old]] [-C] [-Df|b] [%s] [-Lf|b<d_km>]\n", GMT_program, GMT_H_OPT);
-		fprintf (stderr, "\t[%s] [-N<upper_age>] [-Q<t_fix>] [-S<stem>] [-T<t_zero>] [-V] [-W] [%s] [%s]\n\n", GMT_M_OPT, GMT_t_OPT, GMT_b_OPT);
+		fprintf (stderr, "usage: %s [infile(s)] -E<euler.d> OR -eplon/plat/prot [-A[young/old]] [-C] [-Df|b] [%s] [-Lf|b<d_km>]\n", GMT_program, GMT_H_OPT);
+		fprintf (stderr, "\t[-N<upper_age>] [-Q<t_fix>] [-S<stem>] [-T<t_zero>] [-V] [-W] [%s] [%s] [%s]\n\n", GMT_t_OPT, GMT_b_OPT, GMT_m_OPT);
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
 		fprintf (stderr, "\tinfiles (in ASCII or binary) has 3 or more columns.  If no file(s) is given, standard input is read.\n");
 		fprintf (stderr, "\tFirst 3 columns must have lon, lat (or lat, lon, see -:) and age (Ma)\n");
-		fprintf (stderr, "\t-E specifies the stage pole file to be used\n\n");
+		fprintf (stderr, "\t-E specifies the rotations to be used (see man page for format)\n\n");
+		fprintf (stderr, "\t-e Alternatively, specify a single finite rotation (in degrees) to be applied to all input points\n");
 		fprintf (stderr, "\tOPTIONS:\n\n");
 		fprintf (stderr, "\t-A Output tracks for ages (or stages, see -L) between young and old [Default is entire track]\n");
 		fprintf (stderr, "\t   If no limit is given, then each seamount should have their limits in columns 4 and 5 instead.\n");
 		fprintf (stderr, "\t   Only applicable in conjunction with the -L option.\n");
 		fprintf (stderr, "\t-C The file given with -E contains finite rotation poles [Default is stage poles]\n");
-		fprintf (stderr, "\t-Db move backward in time (from older to younger positions) [Default]\n");
-		fprintf (stderr, "\t-Df move foreward in time (from younger to older positions)\n");
+		fprintf (stderr, "\t-Db Backtrack mode: move forward in time (from older to younger positions) [Default]\n");
+		fprintf (stderr, "\t-Df Flowline mode: move backward in time (from younger to older positions)\n");
 		GMT_explain_option ('H');
 		fprintf (stderr, "\t-Lb Compute hotspot tracks sampled every <d_km> interval [Default projects single points].\n");
 		fprintf (stderr, "\t-Lf Compute flowline for seamounts of unknown but maximum age [Default projects single points]\n");
 		fprintf (stderr, "\t    If no <d_km> is given, the start/stop points for each stage are returned\n");
 		fprintf (stderr, "\t    If B and F is used instead, stage id is returned as z-value [Default is predicted ages]\n");
-		GMT_explain_option ('M');
 		fprintf (stderr, "\t-N extends earliest stage pole back to <upper_age> [no extension]\n");
 		fprintf (stderr, "\t-Q Assigned a fixed age to all input points.\n");
 		fprintf (stderr, "\t-S writes tracks to individual files <stem>.# instead of to stdout (requires -L)\n");
@@ -277,18 +290,20 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t   Output record will be lon,lat,az,major,minor.\n");
 		fprintf (stderr, "\t   -Wt will output lon,lat,time,az,major,minor.\n");
 		fprintf (stderr, "\t   -Wa will output lon,lat,angle,az,major,minor.\n");
+		fprintf (stderr, "\t   Use -D to specify which direction to rotate [forward in time].\n");
 		GMT_explain_option (':');
 		GMT_explain_option ('i');
 		GMT_explain_option ('n');
 		fprintf (stderr, "\t   Default is 3 input columns\n");
 		GMT_explain_option ('o');
 		GMT_explain_option ('.');
-		fprintf (stderr, "\t   Output produced by -Lf|t can be plotted with psxy using the -M option\n");
+		fprintf (stderr, "\t   Output produced by -Lf|t can be plotted with psxy using the -m option\n");
+		GMT_explain_option ('m');
 		exit (EXIT_FAILURE);
 	}
 
-	if (confidence && (forward || flowline)) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR.  -W cannot be set if -Lf or -Df are set\n", GMT_program);
+	if (confidence && do_track) {
+		fprintf (stderr, "%s: GMT SYNTAX ERROR.  -W cannot be set if -Lf or -Lb are set\n", GMT_program);
 		error++;
 	}
 	if (GMT_io.binary[GMT_IN] && GMT_io.io_header[GMT_IN]) {
@@ -303,25 +318,27 @@ int main (int argc, char **argv)
 
 	if (error) exit (EXIT_FAILURE);
 
-	/* Load in the stage poles */
-
-	n_stages = spotter_init (euler_file, &p, flowline, finite, finite_out, &upper_age, gmtdefs.verbose);
-
 	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
-		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
+		fprintf (stderr, "%s: Expects %ld-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 
-	spot_func = ((flowline + forward) == 1) ? spotter_forthtrack : spotter_backtrack;
-
-	if (fabs (d_km) > GMT_SMALL) {		/* User wants to interpolate tracks rather than project individual points */
-		make_path = TRUE;
-		(flowline) ? sprintf (type, "Flowline") : sprintf (type, "Hotspot track");
-		(forward) ? sprintf (dir, "from") : sprintf (dir, "to");
+	if (single_rotation) {	/* Get rotation matrix R */
+		spotter_make_rot_matrix (rot_lon, rot_lat, rot_w, R);
 	}
-	else if (track_limit > 0) {	/* Limits on track set but track option not selected */
-                fprintf (stderr, "%s: GMT SYNTAX ERROR.  -A requires -L\n", GMT_program);
-		exit (EXIT_FAILURE);
+	else {	/* Load in the stage poles */
+		n_stages = spotter_init (euler_file, &p, flowline, finite, finite_out, &upper_age, gmtdefs.verbose);
+		spot_func = ((flowline + forward) == 1) ? spotter_forthtrack : spotter_backtrack;
+
+		if (fabs (d_km) > GMT_SMALL) {		/* User wants to interpolate tracks rather than project individual points */
+			make_path = TRUE;
+			(flowline) ? sprintf (type, "Flowline") : sprintf (type, "Hotspot track");
+			(forward) ? sprintf (dir, "from") : sprintf (dir, "to");
+		}
+		else if (track_limit > 0) {	/* Limits on track set but track option not selected */
+	                fprintf (stderr, "%s: GMT SYNTAX ERROR.  -A requires -L\n", GMT_program);
+			exit (EXIT_FAILURE);
+		}
 	}
 
 	fpo = GMT_stdout;
@@ -340,6 +357,7 @@ int main (int argc, char **argv)
 	n_expected_fields = (GMT_io.ncol[GMT_IN]) ? GMT_io.ncol[GMT_IN] : 3 + ((track_limit == 2) ? 2 : 0);
 	if (fix_all && !GMT_io.ncol[GMT_IN]) n_expected_fields = 2;	/* Lon, lat only; use fixed t */
 	if (confidence) n_out = 5 + !(conf_flag == 0);
+	if (single_rotation && !GMT_io.binary[GMT_IN]) n_expected_fields = BUFSIZ;	/* Allow any input for -F mode */
 
 	for (fno = 1; !done && fno < n_args; fno++) {	/* Loop over input files, if any */
 		if (!nofile && argv[fno][0] == '-') continue;
@@ -358,7 +376,7 @@ int main (int argc, char **argv)
 
 		if (GMT_io.io_header[GMT_IN]) {
 			for (i = 0; i < GMT_io.n_header_recs; i++) {
-				GMT_fgets (buffer, BUFSIZ, fp);
+				not_used = GMT_fgets (buffer, BUFSIZ, fp);
 				if (first && !GMT_io.binary[GMT_OUT] && GMT_io.io_header[GMT_OUT]) fprintf (GMT_stdout, "%s", buffer);
 			}
 			first = FALSE;
@@ -374,17 +392,27 @@ int main (int argc, char **argv)
 			if (GMT_io.status & GMT_IO_EOF) continue;
 
 			if (GMT_io.status & GMT_IO_MISMATCH) {
-				fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d (skipped)\n", GMT_program, n_fields, n_expected_fields, n_read);
+				fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld (skipped)\n", GMT_program, n_fields, n_expected_fields, n_read);
 				continue;
 			}
+			
+			if (!out) out = (double *) GMT_memory (VNULL, (size_t)MAX(n_out, n_expected_fields), sizeof (double), GMT_program);
+
+			if (single_rotation) {	/* Simple reconstruction, then exit */
+				GMT_geo_to_cart (in[GMT_Y], in[GMT_X], x, TRUE);	/* Get x-vector */
+				spotter_matrix_vect_mult (R, x, y);			/* Rotate the x-vector */
+				GMT_cart_to_geo (&out[GMT_Y], &out[GMT_X], y, TRUE);	/* Recover lon lat representation; TRUE to get degrees */
+				memcpy ((void *)&out[GMT_Z], (void *)&in[GMT_Z], (n_fields - 2) * sizeof (double));
+				GMT_output (fpo, n_fields, out);
+				continue;
+			}
+			
 			if (fix_all) in[2] = t_fix;
 			if (in[2] < 0.0) {	/* Negative ages are flags for points to be skipped */
 				n_skipped++;
 				continue;
 			}
 
-			if (!out) out = (double *) GMT_memory (VNULL, (size_t)MAX(n_out, n_expected_fields), sizeof (double), GMT_program);
-
 			lon = in[0] * D2R;
 			lat = in[1] * D2R;
 			if (track_limit) {
@@ -395,7 +423,7 @@ int main (int argc, char **argv)
 				age = in[2];
 
 			if (age > upper_age) {	/* Points older than oldest stage cannot be used */
-				fprintf (stderr, "%s: Seamount at line %d has age (%g) > oldest stage (%g) (skipped)\n", GMT_program, n_read, in[2], upper_age);
+				fprintf (stderr, "%s: Seamount at line %ld has age (%g) > oldest stage (%g) (skipped)\n", GMT_program, n_read, in[2], upper_age);
 				n_skipped++;
 				continue;
 			}
@@ -404,7 +432,7 @@ int main (int argc, char **argv)
 
 			if (make_path) {	/* Asked for paths, now write out several multiple segment tracks */
 				if (seg_files) {
-					sprintf (buffer, "%s.%d", file_stem, n_points);
+					sprintf (buffer, "%s.%ld", file_stem, n_points);
 					if ((fpo = GMT_fopen (buffer, GMT_io.w_mode)) == NULL) {
 						fprintf (stderr, "%s: Error - cannot create file %s\n", GMT_program, buffer);
 						exit (EXIT_FAILURE);
@@ -428,7 +456,7 @@ int main (int argc, char **argv)
 			}
 			else {	/* Just return the projected locations */
 				if (confidence) {	/* Asked for confidence ellipses on reconstructed points */
-					if (spotter_conf_ellipse (in[0], in[1], age, p, n_stages, conf_flag, out)) {
+					if (spotter_conf_ellipse (in[0], in[1], age, p, n_stages, conf_flag, forward, out)) {
 						fprintf (stderr, "%s: Confidence ellipses only for the age of rotations.  Point with age %g skipped\n", GMT_program, age);
 						continue;
 					}
@@ -449,15 +477,18 @@ int main (int argc, char **argv)
 
 	if (gmtdefs.verbose) {
 		if (make_path)
-			fprintf (stderr, "%s: %d segments written\n", GMT_program, n_points);
+			fprintf (stderr, "%s: %ld segments written\n", GMT_program, n_points);
 		else
-			fprintf (stderr, "%s: %d points projected\n", GMT_program, n_points);
+			fprintf (stderr, "%s: %ld points projected\n", GMT_program, n_points);
 	}
 
-	if (gmtdefs.verbose && n_skipped) fprintf (stderr, "%s: %d points skipped because age < 0\n", GMT_program, n_skipped);
+	if (gmtdefs.verbose && n_skipped) fprintf (stderr, "%s: %ld points skipped because age < 0\n", GMT_program, n_skipped);
 
 	/* Clean up and exit */
 
+	if (!single_rotation) GMT_free ((void *)p);
+	if (out) GMT_free ((void *)out);
+	
 	GMT_end (argc, argv);
 
 	exit (EXIT_SUCCESS);
