@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------
- *	$Id: libspotter.c,v 1.54 2009/01/09 04:02:35 guru Exp $
+ *	$Id: libspotter.c,v 1.57 2009/05/13 21:06:44 guru Exp $
  *
  *   Copyright (c) 1999-2009 by P. Wessel
  *
@@ -223,9 +223,7 @@ int spotter_hotspot_init (char *file, struct HOTSPOT **p)
 			e[i].fit = (fit == 'Y');
 			e[i].plot = (plot == 'Y');
 		}
-		e[i].lonr = D2R * e[i].lon;
-		e[i].latr = D2R * e[i].lat;
-		GMT_geo_to_cart (&e[i].latr, &e[i].lonr, P, FALSE);
+		GMT_geo_to_cart (e[i].lat, e[i].lon, P, TRUE);
 		e[i].x = P[0];
 		e[i].y = P[1];
 		e[i].z = P[2];
@@ -956,7 +954,7 @@ void spotter_make_rot_matrix (double lonp, double latp, double w, double R[3][3]
 
 	double E[3];
 
-        GMT_geo_to_cart (&latp, &lonp, E, TRUE);
+        GMT_geo_to_cart (latp, lonp, E, TRUE);
 	make_rot_matrix_sub (E, w, R);
 }
 
@@ -1001,7 +999,7 @@ void make_rot0_matrix (double lonp, double latp, double R[3][3], double E[])
 	 *	R		the rotation matrix without terms depending on omega
 	 */
 
-        GMT_geo_to_cart (&latp, &lonp, E, TRUE);
+        GMT_geo_to_cart (latp, lonp, E, TRUE);
 
 	R[0][0] = E[0] * E[0];
 	R[0][1] = E[0] * E[1];
@@ -1095,10 +1093,10 @@ void matrix_to_pole (double T[3][3], double *plon, double *plat, double *w)
 	H = sqrt (H);
 	tr = T[0][0] + T[1][1] + T[2][2];
 
-	*plon = atan2 (T13_m_T31, T32_m_T23) * R2D;
+	*plon = atan2d (T13_m_T31, T32_m_T23);
 	if (*plon < 0.0) (*plon) += 360.0;
-	*plat = atan2 (T21_m_T12, H) * R2D;
-	*w = atan2 (L, (tr - 1.0)) * R2D;
+	*plat = atan2d (T21_m_T12, H);
+	*w = atan2d (L, tr - 1.0);
 	if (*plat < 0.0) {	/* Make N hemisphere pole */
 		*plat = -(*plat);
 		*(plon) += 180.0;
@@ -1135,8 +1133,8 @@ void xyw_to_struct_euler (struct EULER *p, double lon[], double lat[], double w[
 		p[i].omega = w[i];
 		if (convert) p[i].omega /= p[i].duration;	/* Convert opening angle to opening rate */
 		p[i].omega_r = p[i].omega * D2R;
-		p[i].sin_lat = sin (p[i].lat * D2R);
-		p[i].cos_lat = cos (p[i].lat * D2R);
+		p[i].sin_lat = sind (p[i].lat);
+		p[i].cos_lat = cosd (p[i].lat);
 		p[i].lon_r = p[i].lon * D2R;
 		p[i].lat_r = p[i].lat * D2R;
 	}
@@ -1164,7 +1162,7 @@ int spotter_conf_ellipse (double lon, double lat, double t, struct EULER *p, GMT
 	 */
 
 	GMT_LONG matrix_dim = 3L;
-	int i, j, k, kk = 3, nrots;
+	GMT_LONG i, j, k, kk = 3, nrots;
 	double R[3][3], x[3], y[3], M[3][3], RMt[3][3], Rt[3][3], MRt[3][3], cov[3][3], tmp[3][3], C[9];
 	double z_unit_vector[3], EigenValue[3], EigenVector[9], work1[3], work2[3], x_in_plane[3], y_in_plane[3];
 	double x_comp, y_comp, w;
@@ -1206,7 +1204,7 @@ int spotter_conf_ellipse (double lon, double lat, double t, struct EULER *p, GMT
 
 	/* Get projected point y = R*x */
 
-	GMT_geo_to_cart (&lat, &lon, x, TRUE);
+	GMT_geo_to_cart (lat, lon, x, TRUE);
 	for (i = 0; i < 3; i++) y[i] = R[i][0] * x[0] + R[i][1] * x[1] + R[i][2] * x[2];
         GMT_cart_to_geo (&out[1], &out[0], y, TRUE);
 	if (flag == 't')
@@ -1225,7 +1223,7 @@ int spotter_conf_ellipse (double lon, double lat, double t, struct EULER *p, GMT
 	GMT_cross3v (y, x_in_plane, y_in_plane);	/* Local y-axis in plane normal to mean pole */
 	x_comp = GMT_dot3v (EigenVector, x_in_plane);	/* x-component of major axis in tangent plane */
 	y_comp = GMT_dot3v (EigenVector, y_in_plane);	/* y-component of major axis in tangent plane */
-	out[kk] = fmod (360.0 + (90.0 - atan2 (y_comp, x_comp) * R2D), 360.0);	/* Azimuth of major axis */
+	out[kk] = fmod (360.0 + (90.0 - atan2d (y_comp, x_comp)), 360.0);	/* Azimuth of major axis */
 	if (out[kk] > 180.0) out[kk] -= 180.0;
 	out[++kk] = sqrt (EigenValue[0]) * EQ_RAD * SQRT_CHI2;
 	out[++kk] = sqrt (EigenValue[1]) * EQ_RAD * SQRT_CHI2;
@@ -1236,7 +1234,7 @@ int spotter_conf_ellipse (double lon, double lat, double t, struct EULER *p, GMT
 void spotter_set_M (double lon, double lat, double M[3][3])
 {	/* Just initializes the M(x), the skew-symmetric matrix needed to compute cov of rotated point */
 	double x[3];
-        GMT_geo_to_cart (&lat, &lon, x, TRUE);	/* Get Cartesian vector for this point */
+        GMT_geo_to_cart (lat, lon, x, TRUE);	/* Get Cartesian vector for this point */
 	M[0][0] = M[1][1] = M[2][2] = 0.0;
 	M[0][1] = -x[2];
 	M[0][2] = x[1];
@@ -1294,4 +1292,4 @@ void spotter_cov_of_inverse (struct EULER *e, double Ct[3][3])
 	spotter_matrix_transpose (At, A);	/* Get A' */
 	spotter_matrix_mult (e->C, At, tmp);	/* Calculate the cov(u)*A' product */
 	spotter_matrix_mult (A, tmp, Ct);	/* Calculate the cov(v) = A*cov(u)*A' product */
-}
\ No newline at end of file
+}
