@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------
- *	$Id: originator.c,v 1.49 2009/01/09 04:02:36 guru Exp $
+ *	$Id: originator.c,v 1.51 2009/07/11 03:16:32 guru Exp $
  *
  *   Copyright (c) 2000-2009 by P. Wessel
  *
@@ -111,11 +111,11 @@ int main (int argc, char **argv)
 	BOOLEAN	error = FALSE, done, nofile = TRUE, truncate_ages = FALSE, first = TRUE, better, use_ID = FALSE;
 	BOOLEAN finite = FALSE, degree = FALSE;		/* fitite is TRUE if stage pole file contains finite rotation poles instead */
 
-	double x_smt, y_smt, z_smt, r_smt, t_smt, *c, *in, d_km = 5.0, upper_age = 180.0, dist, plon, plat, out[5];
+	double x_smt, y_smt, z_smt, r_smt, t_smt, *c, *in, d_km = 5.0, upper_age = 180.0, dist, out[5];
 	double max_dist = 1.0e100, hx_dist, hx_dist_km, dist_NA, dist_NX, del_dist, dt = 0.0, A[3], H[3], N[3], X[3];
 	double dlon, fix_r, fix_t;
 
-	char *hsfile = CNULL, *efile = CNULL, age[GMT_TEXT_LEN];
+	char *hsfile = CNULL, *efile = CNULL, age[GMT_TEXT_LEN], *not_used = NULL;
 
 	FILE *fp;
 
@@ -326,7 +326,7 @@ int main (int argc, char **argv)
 		n_read = 0;
 		if (GMT_io.io_header[GMT_IN]) {
 			for (i = 0; i < GMT_io.n_header_recs; i++) {
-				GMT_fgets (GMT_io.segment_header, BUFSIZ, fp);
+				not_used = GMT_fgets (GMT_io.segment_header, BUFSIZ, fp);
 				if (first && !GMT_io.binary[GMT_OUT] && GMT_io.io_header[GMT_OUT]) fprintf (GMT_stdout, "%s", GMT_io.segment_header);
 				n_read++;
 			}
@@ -394,16 +394,15 @@ int main (int argc, char **argv)
 			}
 			for (j = 0; j < nh; j++) {
 
-				plon = hot[j].h->lon;	plat = hot[j].h->lat;
-				GMT_geo_to_cart (&plat, &plon, H, TRUE);	/* 3-D Cartesian vector of this hotspot */
+				GMT_geo_to_cart (hot[j].h->lat, hot[j].h->lon, H, TRUE);	/* 3-D Cartesian vector of this hotspot */
 
 				/* Fine-tune the nearest point by considering intermediate points along greatcircle between knot points */
 
 				k = 3 * hot[j].nearest + 1;			/* Corresponding index for x into the (x,y,t) array c */
-				GMT_geo_to_cart (&c[k+1], &c[k], N, FALSE);	/* 3-D vector of nearest node to this hotspot */
+				GMT_geo_to_cart (c[k+1], c[k], N, FALSE);	/* 3-D vector of nearest node to this hotspot */
 				better = FALSE;
 				if (hot[j].nearest > 0) {	/* There is a point along the flowline before the nearest node */
-					GMT_geo_to_cart (&c[k-2], &c[k-3], A, FALSE);	/* 3-D vector of end of this segment */
+					GMT_geo_to_cart (c[k-2], c[k-3], A, FALSE);	/* 3-D vector of end of this segment */
 					if (GMT_great_circle_intersection (A, N, H, X, &hx_dist) == 0) {	/* X is between A and N */
 						hx_dist_km = d_acos (hx_dist) * KM_PR_RAD;
 						if (hx_dist_km < hot[j].np_dist) {	/* This intermediate point is even closer */
@@ -418,7 +417,7 @@ int main (int argc, char **argv)
 					}
 				}
 				if (hot[j].nearest < (np-1) ) {	/* There is a point along the flowline after the nearest node */
-					GMT_geo_to_cart (&c[k+4], &c[k+3], A, FALSE);	/* 3-D vector of end of this segment */
+					GMT_geo_to_cart (c[k+4], c[k+3], A, FALSE);	/* 3-D vector of end of this segment */
 					if (GMT_great_circle_intersection (A, N, H, X, &hx_dist) == 0) {	/* X is between A and N */
 						hx_dist_km = d_acos (hx_dist) * KM_PR_RAD;
 						if (hx_dist_km < hot[j].np_dist) {	/* This intermediate point is even closer */
