@@ -1,7 +1,7 @@
 /*-----------------------------------------------------------------
- *	$Id: x2sys_binlist.c,v 1.50 2008/04/12 01:38:49 guru Exp $
+ *	$Id: x2sys_binlist.c,v 1.61 2009/06/25 03:25:07 guru Exp $
  *
- *      Copyright (c) 1999-2008 by P. Wessel
+ *      Copyright (c) 1999-2009 by P. Wessel
  *      See COPYING file for copying and redistribution conditions.
  *
  *      This program is free software; you can redistribute it and/or modify
@@ -36,19 +36,19 @@ struct BINCROSS {
 
 int main (int argc, char **argv)
 {
-	char *TAG = CNULL, d_unit[2];
+	char *TAG = CNULL, **trk_name;
 
-	int i, j, k, k1, ij, ii, jj, nx, bi, bj, start_i, end_i, jump_180, jump_360, dist_flag = 0;
-	int this_bin_i;	/* This i node for bin */
-	int this_bin_j;	/* This j node for bin */
-	int this_bin_ij;/* This bin */
-	int last_bin_i;	/* Previous i node for bin */
-	int last_bin_j;	/* Previous j node for bin */
-	int last_bin_ij;/* Previous bin */
+	GMT_LONG i, j, k, n_tracks, k1, ij, ii, jj, nx, bi, bj, start_i, end_i, jump_180, jump_360;
+	GMT_LONG this_bin_i;	/* This i node for bin */
+	GMT_LONG this_bin_j;	/* This j node for bin */
+	GMT_LONG this_bin_ij;/* This bin */
+	GMT_LONG last_bin_i;	/* Previous i node for bin */
+	GMT_LONG last_bin_j;	/* Previous j node for bin */
+	GMT_LONG last_bin_ij;/* Previous bin */
 	unsigned int nav_flag;
 	size_t n_alloc = 0, nx_alloc = GMT_SMALL_CHUNK;
 
-	BOOLEAN error = FALSE, trackline = FALSE, gap, ea_mode = FALSE;
+	BOOLEAN error = FALSE, trackline = FALSE, gap, ea_mode = FALSE, cmdline_files;
 
 	double **data, *dist_km = NULL, *dist_bin = NULL, dist_scale, x, y, dx, del_x, del_y, y_max = 90.0;
 
@@ -58,14 +58,13 @@ int main (int argc, char **argv)
 	struct BINCROSS *X;
 
 	int outside (double x, double y, struct X2SYS_BIX *B, BOOLEAN geo);
-	unsigned int get_data_flag (double *data[], int j, struct X2SYS_INFO *s);
+	unsigned int get_data_flag (double *data[], GMT_LONG j, struct X2SYS_INFO *s);
 	int comp_bincross (const void *p1, const void *p2);
 
 	argc = GMT_begin (argc, argv);
 	
 	for (i = strlen(argv[0]); i >= 0 && argv[0][i] != '/'; i--);
 	X2SYS_program = &argv[0][i+1];	/* Name without full path */
-	d_unit[0] = 'k';	d_unit[1] = '\0';	/* Default is -Ndk */
 
 	for (i = 1; i < argc; i++) {
 		if (argv[i][0] == '-') {
@@ -79,29 +78,12 @@ int main (int argc, char **argv)
 
 				/* Supplemental parameters */
 
-				case 'C':
-					if (argv[i][2] == 'c') dist_flag = 0;
-					if (argv[i][2] == 'f') dist_flag = 1;
-					if (argv[i][2] == 'g') dist_flag = 2;
-					if (argv[i][2] == 'e') dist_flag = 3;
-					if (dist_flag < 0 || dist_flag > 3) {
-						fprintf(stderr, "%s: ERROR -C: Flag must be c, f, g, or e\n", GMT_program);
-						error++;
-					}
-					break;
 				case 'D':
 					trackline = TRUE;
 					break;
 				case 'E':
 					ea_mode = TRUE;
 					break;
-				case 'N':	/* Set distance units */
-					d_unit[0] = argv[i][2];
-					if (!strchr ("ekmn", (int)d_unit[0])) {
-						fprintf(stderr, "%s: ERROR -N: Unit must be e, k, m, or n\n", GMT_program);
-						error++;
-					}
-					break;
 				case 'T':
 					TAG = &argv[i][2];
 					break;
@@ -114,26 +96,15 @@ int main (int argc, char **argv)
 
 	if (argc == 1 || error || GMT_give_synopsis_and_exit) {
 		fprintf (stderr, "x2sys_binlist %s - Create bin index listing from data files\n\n", X2SYS_VERSION);
-		fprintf (stderr, "usage: x2sys_binlist <files> -T<TAG> [-Cc|f|g|e] [-D] [-E] [-Ne|k|n|m] [-V]\n\n");
+		fprintf (stderr, "usage: x2sys_binlist <files> -T<TAG> [-D] [-E] [-V]\n\n");
 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
-		fprintf (stderr, "\t<files> is one or more datafiles\n");
+		fprintf (stderr, "\t<files> is one or more datafiles, or give =<files.lis> for a file with a list of datafiles\n");
 		fprintf (stderr, "\t-T <TAG> is the system tag for this compilation\n");
 		fprintf (stderr, "\n\tOPTIONS:\n");
-		fprintf (stderr, "\t-C Append code for distance calculation procedure:\n");
-		fprintf (stderr, "\t   c Cartesian [Default]\n");
-		fprintf (stderr, "\t   f Flat Earth\n");
-		fprintf (stderr, "\t   g Great circle\n");
-		fprintf (stderr, "\t   e Ellipsoidal (geodesic) using current GMT ellipsoid\n");
-		fprintf (stderr, "\t   [Default is great-circle or geodesic, depending on ELLIPSOID]\n");
 		fprintf (stderr, "\t-D Calculate track-lengths per bin (see -C for method and -N for units)\n");
 		fprintf (stderr, "\t-E Do binning using equal-area bins (with -D only)\n");
-		fprintf (stderr, "\t-N Append choice for distance unit (assumes -Df|g|e). Choose among:\n");
-		fprintf (stderr, "\t   e Metric units I (meters)\n");
-		fprintf (stderr, "\t   k Metric units II (km) [Default]\n");
-		fprintf (stderr, "\t   m British/US units (miles)\n");
-		fprintf (stderr, "\t   n Nautical units (nautical miles)\n");
 		GMT_explain_option ('V');
 		exit (EXIT_FAILURE);
 	}
@@ -143,8 +114,18 @@ int main (int argc, char **argv)
 		exit (EXIT_FAILURE);		
 	}
 	
+	if ((n_tracks = x2sys_get_tracknames (argc, argv, &trk_name, &cmdline_files)) == 0) {
+		fprintf (stderr, "%s: No datafiles given!\n", GMT_program);
+		exit (EXIT_FAILURE);		
+	}
+
 	x2sys_err_fail (x2sys_set_system (TAG, &s, &B, &GMT_io), TAG);
 
+	if (ea_mode && !s->geographic) {
+		fprintf (stderr, "%s: -E requires geographic data; your TAG implies Cartesian\n", GMT_program);
+		exit (EXIT_FAILURE);		
+	}
+
 	if (s->geographic) {
 		GMT_io.out_col_type[0] = GMT_IS_LON;
 		GMT_io.out_col_type[1] = GMT_IS_LAT;
@@ -154,7 +135,7 @@ int main (int argc, char **argv)
 		GMT_io.out_col_type[0] = GMT_io.out_col_type[1] = GMT_IS_FLOAT;
 	GMT_io.out_col_type[2] = GMT_IS_FLOAT;
 	
-	MGD77_Set_Unit (d_unit, &dist_scale, -1);	/* Gets scale which multiplies meters to chosen distance unit */
+	MGD77_Set_Unit (s->unit[X2SYS_DIST_SELECTION], &dist_scale, -1);	/* Gets scale which multiplies meters to chosen distance unit */
 
 	if (ea_mode) {
 		double mid;
@@ -189,13 +170,11 @@ int main (int argc, char **argv)
 
 	fprintf (GMT_stdout, "# %s\n", TAG);
 
-	for (i = 1; i < argc; i++) {
+	for (i = 0; i < n_tracks; i++) {
 
-		if (argv[i][0] == '-') continue;
+		if (gmtdefs.verbose) fprintf (stderr, "x2sys_binlist: Reading file %s ", trk_name[i]);
 
-		if (gmtdefs.verbose) fprintf (stderr, "x2sys_binlist: Reading file %s ", argv[i]);
-
-		x2sys_err_fail ((s->read_file) (argv[i], &data, s, &p, &GMT_io, &j), argv[i]);
+		x2sys_err_fail ((s->read_file) (trk_name[i], &data, s, &p, &GMT_io, &j), trk_name[i]);
 		if (gmtdefs.verbose) fprintf (stderr, "[%s]\n", s->path);
 		
 		if (ea_mode) {	/* Project coordinates */
@@ -207,7 +186,7 @@ int main (int argc, char **argv)
 		memset ((void *)B.binflag, 0, (size_t)(B.nm_bin * sizeof (unsigned int)));
 		if (trackline) {
 			memset ((void *)dist_bin, 0, (size_t)(B.nm_bin * sizeof (double)));
-			GMT_err_fail (GMT_distances (data[s->x_col], data[s->y_col], p.n_rows, dist_scale, -dist_flag, &dist_km), "");	/* -ve gives increments */
+			GMT_err_fail (GMT_distances (data[s->x_col], data[s->y_col], p.n_rows, dist_scale, -s->dist_flag, &dist_km), "");	/* -ve gives increments */
 		}
 
 		last_bin_ij = last_bin_i = last_bin_j = -1;
@@ -317,7 +296,7 @@ int main (int argc, char **argv)
 					if (s->geographic && fabs (y) > y_max) y = copysign (y_max, y);
 					x2sys_err_fail (x2sys_bix_get_ij (x, y, &ii, &jj, &B, &ij), "");
 					dist_bin[ij] += (GMT_distance_func) (X[k].x, X[k].y, X[k1].x, X[k1].y);
-					B.binflag[ij] |= nav_flag;		/* Only update nav flags we havent been here already */
+					B.binflag[ij] |= nav_flag;		/* Only update nav flags we have not been here already */
 				}
 			}
 			last_bin_ij = this_bin_ij;
@@ -325,11 +304,11 @@ int main (int argc, char **argv)
 			last_bin_j = this_bin_j;
 		}
 
-		x2sys_free_data (data, s->n_fields);
+		x2sys_free_data (data, s->n_fields, &p);
 
 		/* Time for bin index output */
 
-		fprintf (GMT_stdout, "> %s\n", argv[i]);
+		fprintf (GMT_stdout, "> %s\n", trk_name[i]);
 		for (ij = 0; ij < B.nm_bin; ij++) {
 			if (B.binflag[ij] == 0) continue;
 			x = B.x_min + ((ij % B.nx_bin) + 0.5) * B.bin_x;
@@ -337,7 +316,7 @@ int main (int argc, char **argv)
 			GMT_ascii_output_one (GMT_stdout, x, 0);
 			fprintf (GMT_stdout, "%s", gmtdefs.field_delimiter);
 			GMT_ascii_output_one (GMT_stdout, y, 1);
-			fprintf (GMT_stdout, "%s%d%s%u", gmtdefs.field_delimiter, ij, gmtdefs.field_delimiter, B.binflag[ij]);
+			fprintf (GMT_stdout, "%s%ld%s%u", gmtdefs.field_delimiter, ij, gmtdefs.field_delimiter, B.binflag[ij]);
 			if (trackline) {
 				fprintf (GMT_stdout, "%s", gmtdefs.field_delimiter);
 				GMT_ascii_output_one (GMT_stdout, dist_bin[ij], 2);
@@ -352,6 +331,7 @@ int main (int argc, char **argv)
 	x2sys_end (s);
 	GMT_free ((void *)B.binflag);
 	if (trackline) GMT_free ((void *)dist_bin);
+	x2sys_free_list (trk_name, n_tracks);
 
 	GMT_end (argc, argv);
 
@@ -372,7 +352,7 @@ int outside (double x, double y, struct X2SYS_BIX *B, BOOLEAN geo)
 	return (0);	/* Inside */
 }
 
-unsigned int get_data_flag (double *data[], int j, struct X2SYS_INFO *s)
+unsigned int get_data_flag (double *data[], GMT_LONG j, struct X2SYS_INFO *s)
 {
 	int i;
 	unsigned int bit, flag;
